# GNU Solfege - eartraining for GNOME
# Copyright (C) 2000, 2001, 2002  Tom Cato Amundsen
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
"""
TODO:
    - cfg.get_list
    - cfg.set_list
    - watches outside ConfigUtils
>>> import cfg
>>> cfg.initialise("default.config", "~/.cfg-test")
>>> cfg.get_int("gui/button_height")
40
>>> cfg.set_string("section/stringvar", "value")
>>> cfg.get_string("section/stringvar")
'value'
>>> cfg.set_int("section/intvar", 14)
>>> cfg.get_int("section/intvar")
14
>>> cfg.set_float("section/floatvar", 3.141592)
>>> cfg.get_float("section/floatvar") == 3.141592
1
>>> cfg.set_bool("section/boolvar", 4)
>>> cfg.get_bool("section/boolvar")
1
>>> cfg.set_bool("section/boolvar", 0)
>>> cfg.get_bool("section/boolvar")
0
>>> c = ConfigUtils("section")
>>> c.get_string("section/stringvar")
'value'
>>> c.get_int("section/intvar")
14
>>> c.get_bool("section/boolvar")
0
>>> c.get_float("section/floatvar") == 3.141592
1
>>> cfg.initialise(None, "~/.solfege-cfg-testing")
>>> c.get_string("ss/a")
''
>>> c.get_string("ss/a=hei")
'hei'
>>> c.get_int("ss/a=3")
3
>>> c.get_list("ss/a")
[]
>>> c.get_list("ss/a=[1, 2, 3]")
[1, 2, 3]
"""
import string, re
import types
import os, os.path, sys

# It is a know fact that many windows lusers are $HOMEless
if sys.platform == 'win32':
    if not os.path.exists(os.path.expanduser("~")):
        os.makedirs(os.path.expanduser("~"))

section_re = re.compile("\[(.*?)\]")
value_re = re.compile("(.*?)=(.*)")
comment_re = re.compile("#.*")

# FIXME why did I comment out the lines below. Test in ms windows
# if this still works, and if so, delete the commented lines.
def split(key):
   #if "/" in key:
       return string.split(key, "/")
   #else:
   #    return string.split(key, "\\")

def parse_file_into_dict(dict, filename):
    file = open(filename, "r")
    line = file.readline()
    section = None
    while line:
        section_m = section_re.match(line)
        value_m = value_re.match(line)
        comment_m = comment_re.match(line)
        if section_m:
            section = section_m.groups()[0]
            if not dict.has_key(section):
                dict[section] = {}
        elif value_m:
            assert section
            dict[section][value_m.groups()[0]] = value_m.groups()[1]
        elif line == "\n":
            pass
        elif comment_m:
            pass
        else:
           raise "X"
        line = file.readline()
    file.close()
    return dict


def sync():
    dump(data, _user_filename)

def _maybe_create_key(key):
    global data
    section, k = split(key)
    if not data.has_key(section):
        data[section] = {}

########################
## set_XXXX functions ##
########################

def set_string(key, val):
    global data
    _maybe_create_key(key)
    section, k = split(key)
    data[section][k] = val

def set_int(key, val):
    global data
    assert type(val) == types.IntType
    _maybe_create_key(key)
    section, k = split(key)
    data[section][k] = str(val)

def set_float(key, val):
    global data
    assert type(val) == types.FloatType
    _maybe_create_key(key)
    section, k = split(key)
    data[section][k] = str(val)

def set_bool(key, val):
    global data
    _maybe_create_key(key)
    if val:
        set_string(key, "true")
    else:
        set_string(key, "false")


########################
## get_XXX functions ##
########################

def get_string(key):
    if len(string.split(key, "=")) == 2:
        key, default = string.split(key, "=")
    else:
        default = ""
    section, k = split(key)
    try:
        return data[section][k]
    except KeyError:
        return default

def get_int(key):
   if len(string.split(key, "=")) == 2:
      key, default = string.split(key, "=")
   else:
      default = 0
   section, k = split(key)
   try:
       return int(float(data[section][k]))
   except KeyError:
       return int(default)

def get_float(key):
   if len(string.split(key, "=")) == 2:
      key, default = string.split(key, "=")
   else:
      default = 0
   section, k = split(key)
   try:
       return float(data[section][k])
   except KeyError:
       return float(default)


def get_bool(key):
    if get_string(key) == 'true':
        return 1
    elif get_string(key) == 'false':
        return 0
    else:
        print "warning: cfg.get_bool: value of '%s' is not true or false" % key
        return 0

######################################
## reading and writing the database ##
######################################

def dump(dict, fn):
    f = open(fn, 'w')
    for section in dict.keys():
        f.write("[%s]\n" % section)
        for name in dict[section].keys():
            f.write("%s=%s\n" % (name, dict[section][name]))
        f.write("\n")
    f.close()

def drop_user_config():
    global data, _watches, _watch_counter
    data = {}
    _watches = {}
    _watch_counter = 0
    data = parse_file_into_dict(data, _system_filename)

def reread_data():
    global data, _watches, _watch_counter
    _watches = {}
    _watch_counter = 0
    if _app_defaults_filename:
        data = parse_file_into_dict({}, _app_defaults_filename)
    if _system_filename:
        data = parse_file_into_dict(data, _system_filename)
    if _user_filename and os.path.isfile(_user_filename):
        data = parse_file_into_dict(data, _user_filename)

def initialise(app_defaults_filename, system_filename, user_filename):
    """
    app_defaults_filename: file must exist!
    system_filename: sys admin can override app_defaults_filename
    user_filename: the state of the app is stored here, also user config.
    """
    global data, _watches, _watch_counter
    global _app_defaults_filename, _system_filename, _user_filename
    _watches = {}
    _watch_counter = 0
    _app_defaults_filename = app_defaults_filename
    _system_filename = system_filename
    _user_filename = user_filename = os.path.expanduser(user_filename)
    if app_defaults_filename:
        try:
            data = parse_file_into_dict({}, app_defaults_filename)
        except:
            raise "Solfege install is completely broken.\nCannot find required file %s" % app_defaults_filename
    else:
        data = {}
    if system_filename and os.path.isfile(system_filename):
        data = parse_file_into_dict(data, system_filename)
    if os.path.isfile(user_filename):
        data = parse_file_into_dict(data, user_filename)


class ConfigUtils:
    def __init__(self, exname):
        self.m_exname = exname
    def add_watch(self, name, callback):
        global _watches, _watch_counter
        name = self._expand_name(name)
        if not _watches.has_key(name):
            _watches[name] = {}
        _watches[name][_watch_counter] = callback
        _watch_counter = _watch_counter + 1
        return _watch_counter - 1
    def remove_watch(self, name, id):
        global _watches, _watch_counter
        name = self._expand_name(name)
        if _watches.has_key(name):
            if _watches[name].has_key(id):
                del _watches[name][id]
                if _watches[name] == {}:
                    del _watches[name]
            else:
                print "warning: remove_watch: id don't exist:", id
        else:
            print "warning: remove_watch: name is not watched:", name
    def _expand_name(self, name):
        if string.count(name, "/") == 0:
            return "%s/%s" % (self.m_exname, name)
        return name
    def _set(self, func, name, val):
        global _watches
        name = self._expand_name(name)
        oldval = get_string(name)
        func(name, val)
        newval = get_string(name)
        if _watches.has_key(name) and (oldval != newval):
            for cb in _watches[name].values():
               cb(name)
    def set_int(self, name, val):
        self._set(set_int, name, val)
    def set_float(self, name, val):
        self._set(set_float, name, val)
    def set_string(self, name, val):
        self._set(set_string, name, val)
    def set_list(self, name, val):
        self._set(set_string, name, str(val))
    def set_bool(self, name, val):
        self._set(set_bool, name, val)
    def _get(self, func, name, default=""):
        return func(self._expand_name(name)+default)
    def get_int(self, name):
        return self._get(get_int, name)
    def get_int_with_default(self, name, default):
        assert type(default) is type(0)
        return self.get_int(name+"=%i" % default)
    def get_float(self, name):
        return self._get(get_float, name)
    def get_string(self, name):
        r = self._get(get_string, name)
        if r is None:
            return ""
        else:
            return r
    def get_list(self, name):
        if '=' in name:
           return eval(self._get(get_string, name))
        return eval(self._get(get_string, name, "=[]"))
    def get_bool(self, name):
        return self._get(get_bool, name)


if __name__ == '__main__':
    import doctest, cfg
    print doctest.testmod(cfg)



#!/usr/bin/env python

# This file is free software; as a special exception the author gives
# unlimited permission to copy and/or distribute it, with or without
# modifications, as long as this notice is preserved.
#
# This program is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY, to the extent permitted by law; without even the
# implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
"""
I used this to display the saved statistics in $HOME/.solfegeX.Y/
"""
import pickle, sys
for fn in sys.argv[1:]:
    f = open(fn, "r")
    D = pickle.load(f)
    print "Un-pickled file:", fn
    print D
    f.close()



# GNU Solfege - eartraining for GNOME
# Copyright (C) 2000, 2001, 2002  Tom Cato Amundsen
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

import re
import gtk

class HarmonicProgressionLabel(gtk.GtkHBox):
    """
    This class can parse strings like I-(6,4)V(5,3)-I and can be used
    as button labels.
    """
    def __init__(self, str, align=''):
        gtk.GtkHBox.__init__(self)
        self.show()
        self.set_text(str, align)
    def set_text(self, str, align=''):
        for o in self.children():
            o.destroy()
        self.m_str = str
        if align == 'center':
            self.pack_start(gtk.GtkHBox())
        while self.m_str:
            T, A, B = self.get_next_token()
            if T == 'big' or T == 'err':
                self.bigchar(A)
            elif T == 'two':
                self.twoline(A, B)
            else:
                assert T == 'one'
                self.oneline(A)
        if align == 'center':
            self.pack_start(gtk.GtkHBox())
        self.show_all()
    def get_next_token(self):
        m_re1 = re.compile("([^\(]+)")
        m_re2 = re.compile("\((\w*),\s*(\w*)\)")
        m_re3 = re.compile("\((\w*)\)")
        m1 = m_re1.match(self.m_str)
        m2 = m_re2.match(self.m_str)
        m3 = m_re3.match(self.m_str)
        if m1:
            self.m_str = self.m_str[len(m1.group()):]
            return "big", m1.groups()[0], None
        if m2:
            self.m_str = self.m_str[len(m2.group()):]
            return "two", m2.groups()[0], m2.groups()[1]
        if m3:
            self.m_str = self.m_str[len(m3.group()):]
            return "one", m3.groups()[0], None
        return "err"
    def twoline(self, A, B):
        vbox = gtk.GtkVBox()
        t1 = gtk.GtkLabel(A)
        t1.set_name("ProgressionLabelNumber")
        t1.show();vbox.pack_start(t1);
        t1.set_alignment(0, 0);
        t2 = gtk.GtkLabel(B)
        t2.set_name("ProgressionLabelNumber")
        t2.show();vbox.pack_start(t2);
        t2.set_alignment(0, 0);
        self.pack_start(vbox, gtk.FALSE)
    def oneline(self, A):
        vbox = gtk.GtkVBox()
        t = gtk.GtkLabel(A)
        t.set_name("ProgressionLabelNumber")
        t.show();vbox.pack_start(t);
        t.set_alignment(0, 0);
        self.pack_start(vbox, gtk.FALSE)
    def bigchar(self, A):
        t1 = gtk.GtkLabel(A)
        t1.set_name("ProgressionNameLabel")
        t1.show()
        self.pack_start(t1, gtk.FALSE)


class QuestionNameButtonTable(gtk.GtkTable):
    """
    In lack of a better name for this class...
    This gtk.GtkTable extension creates a table width buttons that is added
    in an intelligent way, and the labels of the buttons can be normal
    text or chord progression names. See: HarmonicProgressionLabel
    """
    def __init__(self):
        gtk.GtkTable.__init__(self)
    def initialize(self, num, dir):
        self.m_num = num
        self.m_dir = dir
        self.m_x = 0
        self.m_y = 0
        for X in self.children():
            X.destroy()
    def add(self, key, label_code, style, callback):
        b = gtk.GtkButton()
        b.set_data('pcode', key)
        if style == 'progression':
            b.add(HarmonicProgressionLabel(label_code, 'center'))
        else:	
            b.add(gtk.GtkLabel(label_code))
        b.show_all()
        self.attach(b, self.m_x, self.m_x+1, self.m_y, self.m_y+1)
        b.connect('clicked', callback)
        if self.m_dir == 'vertic':
            self.m_y = self.m_y + 1
            if self.m_y == self.m_num:
                self.m_y = 0
                self.m_x = self.m_x + 1
        else:
            self.m_x = self.m_x + 1
            if self.m_x == self.m_num:
                self.m_x = 0
                self.m_y = self.m_y + 1




# GNU Solfege - eartraining for GNOME
# Copyright (C) 2000, 2001, 2002  Tom Cato Amundsen
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

import types, random

class InvalidNotenameException:
    def __init__(self, n):
        self.m_notename = n
    def __str__(self):
        return self.m_notename

class MusicalPitch:
    LOWEST_STEPS = -28
    HIGHEST_STEPS = 47
    def __init__(self, n=None, a=None, o=None, null_value=0):
        """Will accept five types of arguments:
        0.   n is None, set to "c"
        1.   n is a MusicalPitch. This is necessary to make clones of this
             object.
        2.   n is a string, like cis  Des  g'' Will ignore argument a and o
        3.   n is an integer representing the tone.
        4.   three integers

             c,,,, is lowest: m_octave_i == -4, steps() == -28
             g'''''' is highest: m_octave_i = 6, steps() == 46

        """
        if n is None:
            self.m_notename_i = self.m_accidental_i = self.m_octave_i = 0
        elif isinstance(n, MusicalPitch):
            self.m_notename_i = n.m_notename_i
            self.m_accidental_i = n.m_accidental_i
            self.m_octave_i = n.m_octave_i
        elif type(n) == types.IntType and a is None and o is None:
            self.set_from_midiint(n)
        elif type(n) == types.StringType:
            self.set_from_notename(n)
        else:
            assert type(n) == type(a) == type(o) == types.IntType
            self.m_notename_i = n
            # 0 is natural, 1 is sharp etc.
            self.m_accidental_i = a
            # 0 is c     1 is c'    -1 is c, 
            self.m_octave_i = o
    def transpose_by_musicalpitch(self, P):
        """Silly function used by mpd/parser.py and company
        (d') transposes up one major second.
        """
        tra = P.semitone_pitch() - 60
        old_p = self.semitone_pitch()
        self.m_notename_i = self.m_notename_i + P.m_notename_i
        self.m_accidental_i = self.m_accidental_i + P.m_accidental_i
        if self.m_notename_i > 6:
            self.m_notename_i = self.m_notename_i - 7
            self.m_octave_i = self.m_octave_i + 1
        self.m_octave_i = self.m_octave_i + P.m_octave_i - 1
        if self.semitone_pitch()-old_p < tra:
            self.m_accidental_i = self.m_accidental_i + 1
        elif self.semitone_pitch()-old_p > tra:
            self.m_accidental_i = self.m_accidental_i - 1
        return self
    def steps(self):
        """0 == lille c
        """
        return self.m_notename_i + self.m_octave_i * 7
    def semitone_pitch(self):
        return [0, 2, 4, 5, 7, 9, 11][self.m_notename_i] + \
               self.m_accidental_i + self.m_octave_i * 12 + 48
    def set_from_midiint(self, midiint):
        self.m_octave_i = (midiint-48)/12
        self.m_notename_i = {0:0, 1:0, 2:1, 3:1, 4:2, 5:3, 6:3, 7:4, 8:4,
                             9:5, 10:5, 11:6}[midiint % 12]
        self.m_accidental_i = midiint-(self.m_octave_i+4)*12 \
                              -[0, 2, 4, 5, 7, 9, 11][self.m_notename_i]
    def set_from_notename(self, notename):
        tmp = notename
        self.m_accidental_i = self.m_octave_i = 0
        while notename[-1] in ["'", ","]:
            if notename[-1] == "'":
                self.m_octave_i = self.m_octave_i + 1
            elif notename[-1] == ",":
                self.m_octave_i = self.m_octave_i - 1
            notename = notename[:-1]
        if notename[:2] == 'es':
            notename = 'ees' + notename[2:]
        if notename[:2] == 'as':
            notename = 'aes' + notename[2:]
        while notename[-2:] == 'es':
            self.m_accidental_i = self.m_accidental_i -1
            notename = notename[:-2]
        while notename[-2:] == 'is':
            self.m_accidental_i = self.m_accidental_i + 1
            notename = notename[:-2]
        try:
            self.m_notename_i = ['c', 'd', 'e', 'f', 'g', 'a', 'b'].index(notename)
        except:
           raise InvalidNotenameException(tmp)
    def randomize(self, lowest, highest):
        """
        lowest and highest can be an integer, string or a MusicalPitch instance
        """
        assert type(lowest) == type(highest)
        if type(lowest) == types.StringType:
            lowest = MusicalPitch(lowest).semitone_pitch()
        if type(highest) == types.StringType:
            highest = MusicalPitch(highest).semitone_pitch()
        self.set_from_midiint(random.randint(int(lowest), int(highest)))
        return self
    def __add__(self, i):
        """
        MusicalPitch + integer = MusicalPitch
        MusicalPitch + Intervall = MusicalPitch
        """
        if type(i) == type(0):
            v = self.semitone_pitch()
            assert 0 <= v + i < 128
            return MusicalPitch(v+i)
        elif i.__class__.__name__ == 'Intervall':#isinstance(i, intervall.Intervall):
            r = MusicalPitch(self)
            _p = r.semitone_pitch()
            r.m_notename_i = r.m_notename_i + i.m_intervall * i.m_dir
            r.m_octave_i = r.m_octave_i + r.m_notename_i / 7 + i.m_octave * i.m_dir
            r.m_notename_i = r.m_notename_i % 7
            _diff = r.semitone_pitch() - _p
            r.m_accidental_i = r.m_accidental_i + (i.get_intvalue() - _diff)
            # to avoid notenames like ciscisciscis :
            if r.m_accidental_i > 2:
               r.m_accidental_i = r.m_accidental_i - 2
               r.m_notename_i = r.m_notename_i + 1
               if r.m_notename_i == 7:
                   r.m_notename_i = 0
                   r.m_octave_i = r.m_octave_i + 1
            if r.m_accidental_i < -2:
               r.m_accidental_i = r.m_accidental_i + 2
               r.m_notename_i = r.m_notename_i - 1
               if r.m_notename_i == -1:
                   r.m_notename_i = 6
                   r.m_octave_i = r.m_octave_i - 1
            return r
        else:
            raise "mpd.MusicalPitch.__add__"
    def __sub__(self, i):
        """
        MusicalPitch - MusicalPitch = integer
        MusicalPitch - integer = MusicalPitch
        """
        if isinstance(i, MusicalPitch):
            return self.semitone_pitch() - i.semitone_pitch()
        assert type(i) == types.IntType
        v = self.semitone_pitch()
        assert 0 <= v - i < 128
        return MusicalPitch(v-i)
    def __int__(self):
        return self.semitone_pitch()
    def __cmp__(self, B):
        if (self is None or self is None):
            return -1
        diff = self - B
        if diff < 0:
            return -1
        elif diff > 0:
            return 1
        else:
            return 0
    def __str__(self):
        return "(MusicalPitch %s)" % self.str()
    def notename(self):
        s = ['c', 'd', 'e', 'f', 'g', 'a', 'b'][self.m_notename_i]
        assert -3 < self.m_accidental_i < 3, self.m_accidental_i
        s = s + ['eses', 'es', '', 'is', 'isis'][self.m_accidental_i+2]
        return s
    def str(self):
        s = self.notename()
        if self.m_octave_i < 0:
            s = s + ',' * (-self.m_octave_i)
        if self.m_octave_i > 0:
            s = s + "'" * self.m_octave_i
        return s


if __name__ == "__main__":
    a = MusicalPitch("g")
    b = MusicalPitch("f")
    r = a - b
    print "%s - %s =" % (a, b), r
    r = a - 2
    print "%s - 2 =" % a, r
    r = a + 3
    print "%s + 3 =" % a, r
    print "a < b", a < b
    print "a > b", a > b



# GNU Solfege - eartraining for GNOME
# Copyright (C) 2000, 2001, 2002  Tom Cato Amundsen
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

from i18n import _
import re, string, types, os
import const
import random
import gettext
import mpd, cfg, utils
from soundcard.rat import Rat

import dataparser


predef = {'dictation': 'dictation',
                      'progression': 'progression',
                      'harmony': 'harmony',
                      'sing-chord': 'sing-chord',
                      'chord': 'chord',
                      'id-by-name': 'id-by-name',
                      'satb': 'satb',
                      'horiz': 'horiz',
                      'vertic': 'vertic',
                      'yes': 1,
                      'no': 0,
                      'tempo': (60, 4)}

class _Header:
    def __init__(self, headerdict):
        self.m_headerdict = headerdict
        for s in ('description', 'labelformat', 'title', 'version'):
            if not self.m_headerdict.has_key(s):
                self.m_headerdict[s] = ""
    def __getattr__(self, name):
        if self.m_headerdict.has_key(name):
            return dataparser.get_translated_string(self.m_headerdict, name)
        if name == 'filldir':
            return 'vertic'


class LessonfileCommon:
    def _parse_file(self, filename):
        parser = dataparser.Dataparser(predef, ('tempo',))
        parser.parse_file(filename)
        self._idx = 0
        self.m_transpose = mpd.MusicalPitch("c'")
        H = filter(lambda e: e['blocktype'] == 'header', parser.m_blocks)
        self.header = _Header(H[0])
        self.m_questions = filter(lambda e: e['blocktype'] == 'question',
                                  parser.m_blocks)
        if utils.compare_version_strings(self.header.version, "1.1.3") == -1:
            def convert(q):
                for n in ('music', 'clue_music'):
                    if not q.has_key(n):
                        continue
                    re_old_times = re.compile(r"\\time\s+(\d+)\s*/\s*(\d+);")
                    r = re_old_times.search(q[n])
                    while r:
                        q[n] = q[n][:r.start()]+r"\time %s/%s" % (r.group(1), r.group(2))+q[n][r.end():]
                        r = re_old_times.search(q[n])
                    re_old_key = re.compile(r"\s*\\key\s+([a-z]+)\s*\\(major|minor);")
                    r = re_old_key.search(q[n])
                    while r:
                        q[n] = q[n][:r.start()]+r"\key %s \%s" % (r.group(1), r.group(2))+q[n][r.end():]
                        r = re_old_key.search(q[n])
                    re_old_clef = re.compile(r"\s*\\clef\s+(\w*);")
                    r = re_old_clef.search(q[n])
                    while r:
                        q[n] = q[n][:r.start()]+r" \clef %s" % r.group(1) + q[n][r.end():]
                        r = re_old_clef.search(q[n])
                return q
            map(convert, self.m_questions)
    def select_random_question(self):
        """
        Select a question by random. Set the self.m_transpose variable if
        the lessonfile say the question should be transposed.
        """
        old_music = self.get_music()
        while 1:
            self._idx = random.randint(0, len(self.m_questions) - 1)
            if self.header.random_transpose == 1:
                self.m_transpose = mpd.MusicalPitch().randomize("g", "fis'")
            elif type(self.header.random_transpose) == type([]):
                self.m_transpose = mpd.MusicalPitch().randomize(
                  mpd.transpose_notename("c'", self.header.random_transpose[0]),
                  mpd.transpose_notename("c'", self.header.random_transpose[1]))
            if old_music == self.get_music() \
                and (len(self.m_questions) > 1 or self.header.random_transpose):
                continue
            break
    def get_tempo(self):
        return self.m_questions[self._idx]['tempo']
    def get_name(self):
        if self.m_questions[self._idx].has_key('name'):
            return self.m_questions[self._idx]['name']
        return ""
    def get_music(self):
        """
        Some of the lessonfile classes can use this. They override it
        if they have more special needs.
        """
        if self.header.musicformat == 'chord':
            return r"\staff\transpose %s{< %s > }" \
              % (self.m_transpose.str(), self.m_questions[self._idx]['music'])
        s = self.m_questions[self._idx]['music']
        if self.header.random_transpose:
            s = string.replace(s, r'\staff',
               r'\staff\transpose %s' % self.m_transpose.str())
            s = string.replace(s, r'\addvoice',
               r'\addvoice\transpose %s' % self.m_transpose.str())
        return s


class DictationLessonfile(LessonfileCommon):
    def __init__(self, filename):
        self._parse_file(filename)
    def get_breakpoints(self):
        r = []
        if self.m_questions[self._idx].has_key('breakpoints'):
            r = self.m_questions[self._idx]['breakpoints']
            if not type(r) == type([]):
                r = [r]
        r = map(lambda e: Rat(e[0], e[1]), r)
        return r
    def get_clue_end(self):
        if self.m_questions[self._idx].has_key('clue_end'):
            return apply(Rat, self.m_questions[self._idx]['clue_end'])
    def get_clue_music(self):
        if self.m_questions[self._idx].has_key('clue_music'):
            return self.m_questions[self._idx]['clue_music']
    def select_previous(self):
        if self._idx > 0:
            self._idx = self._idx - 1
    def select_next(self):
        if self._idx < len(self.m_questions) -1:
            self._idx = self._idx + 1


class SingChordLessonfile(LessonfileCommon):
    def __init__(self, filename):
        self._parse_file(filename)
    def get_satb_vector(self):
        """
        return transposed notenames using self.m_transpose
        """
        if self.header.random_transpose:
            v = []
            for x in string.split(self.m_questions[self._idx]['music'], '|'):
                v.append((mpd.MusicalPitch(x).transpose_by_musicalpitch(self.m_transpose)).str())
            return v
        return string.split(self.m_questions[self._idx]['music'], '|')
    def get_music(self):
        v = string.split(self.m_questions[self._idx]['music'], '|')
        k = "c \major"
        if self.m_questions[self._idx].has_key('key'):
            k = self.m_questions[self._idx]['key']
        music = r"""
                \staff{ \key %s\stemUp <%s> }
                \addvoice{ \stemDown <%s> }
                \staff{ \key %s\clef bass \stemUp <%s>}
                \addvoice{ \stemDown <%s>}
                """ % (k, v[0], v[1], k, v[2], v[3])
        if self.header.random_transpose:
            music = string.replace(music, r"\staff",
                      r"\staff\transpose %s" % self.m_transpose.str())
            music = string.replace(music, r"\addvoice",
                      r"\addvoice\transpose %s" % self.m_transpose.str())
        return music
    def select_random_question(self):
        key = "c \major"
        if self.m_questions[self._idx].has_key('key'):
            key = self.m_questions[self._idx]['key']
        v = string.split(key, " ")
        k = mpd.MusicalPitch(v[0]).str()
        n = ['des', 'aes', 'ees', 'bes', 'f', 'c',
             'g', 'd', 'a', 'e', 'b', 'fis', 'cis', 'gis']
        if k not in n:
            raise "bad key signature in sing-chord lessonfile"
        i = n.index(k) - 5
        if v[1] == '\minor':
            i = i - 3
        d = {
              0: (1, 0, 0, 0),
             -1: (1, 3, 0, 0),
             -2: (-1, 1, 0, 0),
             -3: ( 1, 2, -1, 0),
             -4: (-1, 2, 0, 0),
             -5: ( 1, 1, -1, 0),
             -6: (-1, 3, 1, 0),
             -7: ( 1, 0, -1, 0),
             -8: ( 1, 3, -1, 0),
             -9: (-1, 1, 1, 0),
             -10: (1, 2, -2, 0),
             }
        for x in range(-10, -0):
            d[-x] = (-d[x][0], d[x][1], d[x][2], d[x][3])

        old_music = self.get_music()
        while 1:
            self._idx = random.randint(0, len(self.m_questions) - 1)
            if self.header.random_transpose:
                self.m_transpose = mpd.MusicalPitch("c'") \
                    + apply(mpd.Intervall()._set, d[random.randint(-5-i, 5-i)])
            if old_music == self.get_music() \
                and (len(self.m_questions) > 1 or self.header.random_transpose):
                continue
            break


class IdByNameLessonfile(LessonfileCommon):
    def __init__(self, filename):
        self.parse_file(filename)
    def parse_file(self, filename):
        self._parse_file(filename)
        self.m_names = {}
        self.m_names_in_file_order = []
        for question in self.m_questions:
            if not question.has_key('name'):
                print "discarding question in the lessonfile '%s'" % filename
                print "because of missing 'name' variable"
                continue
            if not self.m_names.has_key(question['name']):
                self.m_names[question['name']] = \
                        dataparser.get_translated_string(question, 'name')
                self.m_names_in_file_order.append(question['name'])
    def get_untranslated_name(self):
        """
        Return the untranslated name for the selected question.
        """
        return self.m_questions[self._idx]['name']
    def get_name(self):
        """
        Return the translated name for the selected question.
        """
        return self.m_names[self.m_questions[self._idx]['name']]


class ChordLessonfile(LessonfileCommon):
    def __init__(self, filename):
        self.parse_file(filename)
    def parse_file(self, filename):
        self._parse_file(filename)
        # C-name = key, translated-name = value
        self.m_chord_types = {}
        self.m_inversions = []
        self.m_toptones = []
        for question in self.m_questions:
            self.m_chord_types[question['name']] = dataparser.get_translated_string(question, 'name')

            # inversion
            if question.has_key('inversion') \
                    and question['inversion'] not in self.m_inversions:
                self.m_inversions.append(question['inversion'])
            # toptone
            if question.has_key('toptone') \
                    and question['toptone'] not in self.m_toptones:
                self.m_toptones.append(question['toptone'])
    def get_music(self):
        """ Returns the music for the curretly selected question.
        The function will transpose the question if the lessonfile
        header say random_transpose = 1
        """
        if not self.header.random_transpose:
            return self.m_questions[self._idx]['music']
        v = []
        for n in string.split(self.m_questions[self._idx]['music']):
            v.append(mpd.MusicalPitch(n).transpose_by_musicalpitch(
                 self.m_transpose).str())
        return string.join(v)
    def get_inversion(self):
        """
        Return 1 for root position, 3 for first inversion etc.
        Return -1 if no inversion info is set in question.
        """
        if self.m_questions[self._idx].has_key('inversion'):
             return self.m_questions[self._idx]['inversion']
        return -1
    def get_toptone(self):
        if self.m_questions[self._idx].has_key('toptone'):
            return self.m_questions[self._idx]['toptone']
        return -1
    def get_chordtype(self):
        return self.m_questions[self._idx]['name']

class LessonFileManager:
    def __init__(self):
        self.parse()
    def parse(self):
        self.m_db = {}
        self.m_invalid_files = []
        vim_tmpfile_re = re.compile("\..*\.swp")
        for collection in string.split(cfg.get_string("config/lessoncollections")):
            self.m_db[collection] = {}
            if not os.path.isdir(os.path.expanduser(cfg.get_string("lessoncollections/%s" % collection))):
                continue
            # filter out backup files
            v = filter(lambda x, r=vim_tmpfile_re: x[-1:] != '~' and not r.match(x), os.listdir(os.path.expanduser(cfg.get_string("lessoncollections/%s" % collection))))
            for filename in v:
                # since I usually run solfege from the source dir:
                if filename in ('CVS', 'Makefile', 'Makefile.in') \
                  or not os.path.isfile(self.get_complete_filename(collection, filename)):
                    continue
                try:
                    p = dataparser.Dataparser(predef)
                    p.parse_file(self.get_complete_filename(collection, filename), 1)
                except dataparser.DataparserException:
                    self.m_invalid_files.append(filename)
                    continue
                c = p.m_blocks[0]['content']
                if type(c) == types.StringType:
                    c = [c]
                for u in c:
                    if not self.m_db[collection].has_key(u):
                        self.m_db[collection][u] = []
                    self.m_db[collection][u].append(filename)
        for collection in self.m_db.keys():
            for content in self.m_db[collection].keys():
                self.m_db[collection][content].sort()
        self.m_htmldoc = """<html><body><p>This list is updated each time
                you start Solfege. As you can see, Solfege need more
                lessonfiles. <a href="lessonfiles.html">Write one!</a>"""
        for collection in self.m_db.keys():
            self.m_htmldoc = self.m_htmldoc + "<h1>%s</h1>" % collection
            for use in self.m_db[collection].keys():
                self.m_htmldoc = self.m_htmldoc + "<h2>%s</h2>" % use
                for file in self.m_db[collection][use]:
                    P = {const.USE_CHORD: 'chord',
                         const.USE_HARMONY: 'harmonic-progression-dictation',
                         const.USE_ID_BY_NAME: 'id-by-name',
                         const.USE_DICTATION: 'dictation',
                         const.USE_SING_CHORD: 'sing-chord'}[use]
                    self.m_htmldoc = self.m_htmldoc + \
                      "<a href=\"solfege:practise/%s/%s/%s\">%s</a> " % (P, collection, file, file)
        self.m_htmldoc = self.m_htmldoc + "</body></html>"
    def get_collections(self):
        """
        Return a list of available collections. 
        """
        return self.m_db.keys()
    def get_complete_filename(self, catalog, fn):
        """
        Return the absolute path to a file in a given collection
        """
        if not catalog:
            catalog = 'solfege'
        return os.path.expanduser(os.path.join(
         cfg.get_string('lessoncollections/%s' % catalog), fn))
    def get_filenames(self, collection, usetype):
        if self.m_db[collection].has_key(usetype):
            return self.m_db[collection][usetype]
        else:
            return []




# GNU Solfege - eartraining for GNOME
# Copyright (C) 2000, 2001, 2002  Tom Cato Amundsen
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

from track import Track
import types
from soundcardexceptions import SoundInitException
synth = None

def initialise_winsynth(synthnum, verbose_init=0):
    import winsynth
    global synth
    synth = winsynth.WinSynth(synthnum, verbose_init)


def initialise_external_midiplayer(cmdline, verbose_init=0):
    global synth
    import midifilesynth
    synth = midifilesynth.MidiFileSynth(cmdline, verbose_init)


def initialise_devicefile(devicefile, devicenum=0, card_info=None,
                          verbose_init=0):
    global synth
    if devicefile == '/dev/sequencer2' or devicefile == '/dev/music':
        import oss_sequencer2
        synth = oss_sequencer2.OSSSequencer2Synth(devicefile, devicenum, 
                                                  verbose_init)
    else:#if devicefile == '/dev/sequencer':
        if devicefile != '/dev/sequencer':
            print "warning: the device file is unknown. Assuming it is /dev/sequencer - compatible"
        import oss_sequencer
        synth = oss_sequencer.OSSSequencerSynth(devicefile, devicenum, 
                                                card_info, verbose_init)

def initialise_using_fake_synth(verbose_init=None):
    global synth
    import fakesynth
    synth = fakesynth.Synth(verbose_init)

def play_note(patch, notelen, chan, pitch, vel=64):
    assert type(patch) == types.IntType and 0 <= patch < 128
    assert type(notelen) == types.IntType
    assert type(chan) == types.IntType and 0 <= chan < 16
    assert type(pitch) == types.IntType and 0 <= pitch < 128
    assert type(vel)==types.IntType and 0 <= vel < 128
    m = Track()
    m.set_patch(0, patch)
    m.note(notelen, chan, pitch, vel)
    synth.play_track(m)


instrument_sections = (
    'piano',
    'cromatic percussion',
    'organ', 
    'guitar',
    'bass',
    'strings',
    'ensemble',
    'brass',
    'reed',
    'pipe',
    'synth lead',
    'synth pad',
    'synth effects',
    'ethnic',
    'percussive',
    'sound effects')

instrument_names = (
    "acoustic grand", # 0
    "bright acoustic", # 1
    "electric grand", # 2
    "honky-tonk", # 3
    "electric piano 1", # 4
    "electric piano 2", # 5
    "harpsichord", # 6
    "clav", # 7
    "celesta", # 8
    "glockenspiel", # 9
    "music box", # 10
    "vibraphone", # 11
    "marimba", # 12
    "xylophone", # 13
    "tubular bells", # 14
    "dulcimer", # 15
    "drawbar organ", # 16
    "percussive organ", # 17
    "rock organ", # 18
    "church organ", # 19
    "reed organ", # 20
    "accordion", # 21
    "harmonica", # 22
    "concertina", # 23
    "acoustic guitar (nylon)", # 24
    "acoustic guitar (steel)", # 25
    "electric guitar (jazz)", # 26
    "electric guitar (clean)", # 27
    "electric guitar (muted)", # 28
    "overdriven guitar", # 29
    "distorted guitar", # 30
    "guitar harmonics", # 31
    "acoustic bass", # 32
    "electric bass (finger)", # 33
    "electric bass (pick)", # 34
    "fretless bass", # 35
    "slap bass 1", # 36
    "slap bass 2", # 37
    "synth bass 1", # 38
    "synth bass 2", # 39
    "violin", # 40
    "viola", # 41
    "cello", # 42
    "contrabass", # 43
    "tremolo strings", # 44
    "pizzicato strings", # 45
    "orchestral strings", # 46
    "timpani", # 47
    "string ensemble 1", # 48
    "string ensemble 2", # 49
    "synthstrings 1", # 50
    "synthstrings 2", # 51
    "choir aahs", # 52
    "voice oohs", # 53
    "synth voice", # 54
    "orchestra hit", # 55
    "trumpet", # 56
    "trombone", # 57
    "tuba", # 58
    "muted trumpet", # 59
    "french horn", # 60
    "brass section", # 61
    "synthbrass 1", # 62
    "synthbrass 2", # 63
    "soprano sax", # 64
    "alto sax", # 65
    "tenor sax", # 66
    "baritone sax", # 67
    "oboe", # 68
    "english horn", # 69
    "bassoon", # 70
    "clarinet", # 71
    "piccolo", # 72
    "flute", # 73
    "recorder", # 74
    "pan flute", # 75
    "blown bottle", # 76
    "skakuhachi", # 77
    "whistle", # 78
    "ocarina", # 79
    "lead 1 (square)", # 80
    "lead 2 (sawtooth)", # 81
    "lead 3 (calliope)", # 82
    "lead 4 (chiff)", # 83
    "lead 5 (charang)", # 84
    "lead 6 (voice)", # 85
    "lead 7 (fifths)", # 86
    "lead 8 (bass+lead)", # 87
    "pad 1 (new age)", # 88
    "pad 2 (warm)", # 89
    "pad 3 (polysynth)", # 90
    "pad 4 (choir)", # 91
    "pad 5 (bowed)", # 92
    "pad 6 (metallic)", # 93
    "pad 7 (halo)", # 94
    "pad 8 (sweep)", # 95
    "fx 1 (rain)", # 96
    "fx 2 (soundtrack)", # 97
    "fx 3 (crystal)", # 98
    "fx 4 (atmosphere)", # 99
    "fx 5 (brightness)", # 100
    "fx 6 (goblins)", # 101
    "fx 7 (echoes)", # 102
    "fx 8 (sci-fi)", # 103
    "sitar", # 104
    "banjo", # 105
    "shamisen", # 106
    "koto", # 107
    "kalimba", # 108
    "bagpipe", # 109
    "fiddle", # 110
    "shanai", # 111
    "tinkle bell", # 112
    "agogo", # 113
    "steel drums", # 114
    "woodblock", # 115
    "taiko drum", # 116
    "melodic tom", # 117
    "synth drum", # 118
    "reverse cymbal", # 119
    "guitar fret noise", # 120
    "breath noise", # 121
    "seashore", # 122
    "bird tweet", # 123
    "telephone ring", # 124
    "helicopter", # 125
    "applause", # 126
    "gunshot") # 127

# the names are taken directly from the OSS documentation (pdf file)
percussion_names = [
        "Acoustic Bass Drum", # 35
        "Bass Drum 1",
        "Side Stick",
        "Acoustic Snare",
        "Hand Clap",
        "Electric Snare",
        "Low Floor Tom",
        "Closed Hi Hat",
        "High Floor Tom",
        "Pedal Hi Hat",
        "Low Tom",
        "Open HiHat",
        "Low-Mid Tom",
        "Hi-Mid Tom",
        "Crash Cymbal 1",
        "High Tom",
        "Ride Cymbal 1",
        "Chinese Cymbal",
        "Ride Bell",
        "Tambourine",
        "Splash Cymbal",
        "Cowbell",
        "Crash Cymbal 2",
        "Vibraslap",
        "Ride Cymbal 2",
        "Hi Bongo",
        "Low Bongo",
        "Mute Hi Conga",
        "Open High Conga",
        "Low Conga",
        "High Timbale",
        "Low Timbale",
        "High Agogo",
        "Agogo Low",
        "Cabasa",
        "Maracas",
        "Short Whistle",
        "Long Whistle",
        "Short Guiro",
        "Long Guiro",
        "Claves",
        "Hi Wood Block",
        "Low Wood Block",
        "Mute Cuica",
        "Open Cuica",
        "Mute Triangle",
        "Open Triangle"]

def percussionname_to_int(name):
    assert type(name) is types.StringType
    return percussion_names.index(name) + 35

def int_to_percussionname(i):
    assert type(i) is types.IntType
    return percussion_names[i-35]





# GNU Solfege - eartraining for GNOME
# Copyright (C) 2000, 2001, 2002  Tom Cato Amundsen
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

import sys
import gtk, gnome, gnome.ui
import gu, mpd
import configureoutput
import widgets
from instrumentselector import ReadOnlyInstrumentSelector
from i18n import _
import cfg
import soundcard

class ConfigWindow(gnome.ui.GnomePropertyBox, cfg.ConfigUtils):
    def on_destroy(self, widget):
        self.m_app.m_ui.g_config_window.destroy()
        self.m_app.m_ui.g_config_window = None
    def __init__(self, app):
        gnome.ui.GnomePropertyBox.__init__(self)
        cfg.ConfigUtils.__init__(self, 'configwindow')
        self.m_app = app
        self._is_alive = 0
        self.__apply_tag = self.connect('apply', self.on_apply)
        self.connect('help', self.on_help)
        self.connect('destroy', self.on_destroy)
        ########
        # midi #
        ########
        vbox = gtk.GtkVBox()
        self.append_page(vbox, gtk.GtkLabel(_("Midi stuff")))
        hbox = gu.bHBox(vbox, gtk.FALSE)
        hbox.set_border_width(gnome.uiconsts.PAD)

        table = gtk.GtkTable(1, 1, gtk.FALSE)
        hbox.pack_start(table, gtk.FALSE)

        gu.tLabel(table, 0, 1, 0, 1, _("Slow bpm:"), 1.0,
                  xoptions=gtk.FILL, xpadding=gnome.uiconsts.PAD)
        gu.tLabel(table, 0, 1, 1, 2, _("Default bpm:"), 1.0, 0.5,
                  xoptions=gtk.FILL, xpadding=gnome.uiconsts.PAD)
        gu.tLabel(table, 0, 1, 2, 3, _("Arpeggio bpm:"), 1.0, 0.5,
                  xoptions=gtk.FILL, xpadding=gnome.uiconsts.PAD)
        self.g_slow_bpm = gu.tSpinButton(table, 1, 2, 0, 1,
                                         self.get_int('config/slow_bpm'),
                                         10, 500, 1, 10,
                                         self.on_changed)
        self.g_default_bpm = gu.tSpinButton(table, 1, 2, 1, 2,
                                            self.get_int('config/default_bpm'),
                                            10, 500, 1, 10,
                                            self.on_changed)
        self.g_arpeggio_bpm = gu.tSpinButton(table, 1, 2, 2, 3,
                                           self.get_int('config/arpeggio_bpm'),
                                             10, 500, 1, 10, self.on_changed)
        self.g_instrsel = ReadOnlyInstrumentSelector(_("Preferred instrument"),
               'config', 'preferred_instrument', self.on_changed)
        self.g_instrsel.set_border_width(gnome.uiconsts.PAD)
        hbox.pack_start(self.g_instrsel, gtk.FALSE)
        ########
        # user #
        ########
        box = gtk.GtkVBox()
        box.set_border_width(gnome.uiconsts.PAD)
        self.append_page(box, gtk.GtkLabel(_("User")))
        hbox = gu.bHBox(box, gtk.FALSE)

        table = gtk.GtkTable(2, 3, gtk.FALSE)
        hbox.pack_start(table, gtk.FALSE)
        gu.tLabel(table, 0, 1, 0, 1, _("Highest note user can sing:"), 1.0,
                  xoptions=gtk.FILL, xpadding=gnome.uiconsts.PAD_SMALL)
        gu.tLabel(table, 0, 1, 1, 2, _("Lowest note user can sing:"), 1.0,
                  xoptions=gtk.FILL, xpadding=gnome.uiconsts.PAD_SMALL)

        self.g_highest_singable = widgets.SimplestNotenameSpinButton(
            self.get_string('user/highest_pitch'))
        self.g_highest_singable.g_entry.connect('changed', self.on_changed)
        table.attach(self.g_highest_singable, 1, 2 , 0, 1)
        self.g_lowest_singable = widgets.SimplestNotenameSpinButton(
            self.get_string('user/lowest_pitch'))
        self.g_lowest_singable.g_entry.connect('changed', self.on_changed)
        table.attach(self.g_lowest_singable, 1, 2, 1, 2)
        widgets.NotenameRangeController(
                  self.g_lowest_singable, self.g_highest_singable,
                  mpd.LOWEST_NOTENAME, mpd.HIGHEST_NOTENAME)

        hbox = gu.bHBox(box, gtk.FALSE)
        self.g_sex_male = gtk.GtkRadioButton(None, _("Male"))
        self.g_sex_male.connect('clicked', self.on_changed)
        hbox.pack_start(self.g_sex_male, gtk.FALSE)
        self.g_sex_female = gtk.GtkRadioButton(self.g_sex_male, _("Female"))
        self.g_sex_female.connect('clicked', self.on_changed)
        hbox.pack_start(self.g_sex_female, gtk.FALSE)
        if self.get_string('user/sex') == 'female':
            self.g_sex_female.set_active(gtk.TRUE)
        #######
        # gui #
        #######
        vbox = gtk.GtkVBox()
        vbox.set_border_width(gnome.uiconsts.PAD)
        self.append_page(vbox, gtk.GtkLabel(_("Gui")))

        hbox = gu.bHBox(vbox, gtk.FALSE)
        hbox.set_spacing(gnome.uiconsts.PAD)

        frame = gtk.GtkFrame(_("Main toolbar"))
        frame.set_border_width(gnome.uiconsts.PAD)
        hbox.pack_start(frame, gtk.FALSE)
        frame_vbox = gtk.GtkVBox()
        frame_vbox.set_border_width(gnome.uiconsts.PAD)
        frame.add(frame_vbox)
        frame_hbox = gu.bHBox(frame_vbox)

        self.g_toolbar_icons = gu.RadioButton(None, _("Icons"), self.on_changed)
        frame_hbox.pack_start(self.g_toolbar_icons, gtk.FALSE)
        self.g_toolbar_text = gu.RadioButton(self.g_toolbar_icons, _("Text"), self.on_changed)
        frame_hbox.pack_start(self.g_toolbar_text, gtk.FALSE)
        self.g_toolbar_both = gu.RadioButton(self.g_toolbar_icons, _("Both"), self.on_changed)
        frame_hbox.pack_start(self.g_toolbar_both, gtk.FALSE)
        sep = gtk.GtkHSeparator()
        frame_vbox.pack_start(sep)
        self.__dict__['g_toolbar_'+self.get_string('gui/toolbar_style=icons')].set_active(gtk.TRUE)
        frame_hbox = gu.bHBox(frame_vbox, gtk.FALSE)
        self.g_toolbar_show = gu.RadioButton(None, _("Show"), self.on_changed)
        frame_hbox.pack_start(self.g_toolbar_show, gtk.FALSE)
        self.g_toolbar_hide = gu.RadioButton(self.g_toolbar_show, _("Hide"), self.on_changed)
        frame_hbox.pack_start(self.g_toolbar_hide, gtk.FALSE)
        if self.get_bool('gui/toolbar_visible'):
            self.g_toolbar_show.set_active(gtk.TRUE)
        else:
            self.g_toolbar_hide.set_active(gtk.TRUE)

        frame = gtk.GtkFrame(_("Navigation toolbar"))
        frame.set_border_width(gnome.uiconsts.PAD)
        hbox.pack_start(frame, gtk.FALSE)
        frame_vbox = gtk.GtkVBox()
        frame_vbox.set_border_width(gnome.uiconsts.PAD)
        frame.add(frame_vbox)
        frame_hbox = gu.bHBox(frame_vbox)
        sep = gtk.GtkHSeparator()
        frame_vbox.pack_start(sep)

        self.g_navbar_icons = gu.RadioButton(None, _("Icons"), self.on_changed)
        frame_hbox.pack_start(self.g_navbar_icons, gtk.FALSE)
        self.g_navbar_text = gu.RadioButton(self.g_navbar_icons, _("Text"), self.on_changed)
        frame_hbox.pack_start(self.g_navbar_text, gtk.FALSE)
        self.g_navbar_both = gu.RadioButton(self.g_navbar_icons, _("Both"), self.on_changed)
        frame_hbox.pack_start(self.g_navbar_both, gtk.FALSE)
        self.__dict__['g_navbar_'+self.get_string('gui/navbar_style=icons')].set_active(gtk.TRUE)
        frame_hbox = gu.bHBox(frame_vbox, gtk.FALSE)
        self.g_navbar_show = gu.RadioButton(None, _("Show"), self.on_changed)
        frame_hbox.pack_start(self.g_navbar_show, gtk.FALSE)
        self.g_navbar_hide = gu.RadioButton(self.g_navbar_show, _("Hide"), self.on_changed)
        frame_hbox.pack_start(self.g_navbar_hide, gtk.FALSE)
        if self.get_bool('gui/navbar_visible'):
            self.g_navbar_show.set_active(gtk.TRUE)
        else:
            self.g_navbar_hide.set_active(gtk.TRUE)

        self.g_mainwin_user_resizeable = gtk.GtkCheckButton(
              _("User resizeable main window"))
        self.g_mainwin_user_resizeable.connect('clicked', self.on_changed)
        self.g_mainwin_user_resizeable.set_active(
                        self.get_bool('gui/mainwin_user_resizeable'))
        vbox.pack_start(self.g_mainwin_user_resizeable, gtk.FALSE)

        self.g_development_menu_visible = gtk.GtkCheckButton(
            _("Show development menu"))
        self.g_development_menu_visible.connect('clicked', self.on_changed)
        self.g_development_menu_visible.set_active(
                       self.get_bool('gui/development_menu_visible'))
        vbox.pack_start(self.g_development_menu_visible, gtk.FALSE)
        self._is_alive = 1

        hbox = gu.bHBox(vbox)
        hbox.set_spacing(gnome.uiconsts.PAD_SMALL)
        hbox.pack_start(gtk.GtkLabel(_("Web browser:")), gtk.FALSE)
        self.g_web_browser = gtk.GtkEntry()
        self.g_web_browser.set_text(self.get_string("config/web_browser"))
        self.g_web_browser.connect('changed', self.on_changed)
        hbox.pack_start(self.g_web_browser)

        hbox = gu.bHBox(vbox)
        hbox.set_spacing(gnome.uiconsts.PAD_SMALL)
        hbox.pack_start(gtk.GtkLabel(_("Mail program:")), gtk.FALSE)
        self.g_mail_program = gtk.GtkEntry()
        self.g_mail_program.set_text(self.get_string("config/mua"))
        self.g_mail_program.connect('changed', self.on_changed)
        hbox.pack_start(self.g_mail_program)

        ############
        # Practise #
        ############
        vbox = gtk.GtkVBox()
        vbox.set_border_width(gnome.uiconsts.PAD)
        self.append_page(vbox, gtk.GtkLabel(_("Practise")))

        self.g_picky_on_new_question = gtk.GtkCheckButton(_("Not allow new question before the old is solved"))
        self.g_picky_on_new_question.set_active(self.get_bool('config/picky_on_new_question'))
        vbox.pack_start(self.g_picky_on_new_question, gtk.FALSE)

        self.g_autorepeat_if_wrong = gtk.GtkCheckButton(_("Repeat question if the answer was wrong"))
        self.g_autorepeat_if_wrong.set_active(self.get_bool('config/auto_repeat_question_if_wrong_answer'))
        vbox.pack_start(self.g_autorepeat_if_wrong, gtk.FALSE)

        self.g_picky_on_new_question.connect('clicked', self.on_changed)
        self.g_autorepeat_if_wrong.connect('clicked', self.on_changed)

        #########
        # sound #
        #########
        if sys.platform == 'win32':
            self.create_win32_sound_page()
        else:
            self.create_linux_sound_page()
        self.show_all()
    def create_linux_sound_page(self):
        vbox = gtk.GtkVBox()
        vbox.set_border_width(gnome.uiconsts.PAD)
        self.append_page(vbox, gtk.GtkLabel(_("Sound setup")))
        txt = gtk.GtkLabel(_("""Solfege has two ways to play music. It can use /dev/music or an external midi player program. The preferred way is to use /dev/music since this offers features that will be utilized more in later versions of this program."""))
        txt.set_line_wrap(1)
        txt.set_justify(gtk.JUSTIFY_FILL)
        txt.set_alignment(0.0, 0.0)
        vbox.pack_start(txt, gtk.FALSE)
        self.g_fakesynth_radio = gu.RadioButton(None, _("No sound"), None)
        vbox.pack_start(self.g_fakesynth_radio, gtk.FALSE)
        hbox = gu.bHBox(vbox, gtk.FALSE)
        self.g_device_radio = gu.RadioButton(self.g_fakesynth_radio,
              _("Use device"), None)
        hbox.pack_start(self.g_device_radio, gtk.FALSE)
        combo = gtk.GtkCombo()
        combo.set_popdown_strings(('/dev/sequencer', '/dev/sequencer2',
                                   '/dev/music'))
        self.g_device_entry = combo.entry
        self.g_device_entry.set_text(self.get_string('sound/device_file'))
        self.g_synth_num = gtk.GtkSpinButton(gtk.GtkAdjustment(0, 0, 100, 1, 1),
                             digits=0)
        self.g_synth_num.set_value(self.get_int('sound/synth_number'))
        hbox.pack_start(combo, gtk.FALSE)
        hbox.pack_start(self.g_synth_num, gtk.FALSE)

        hbox = gu.bHBox(vbox, gtk.FALSE)
        self.g_midiplayer_radio = gu.RadioButton(self.g_fakesynth_radio,
             _("Use external midiplayer"), None)
        hbox.pack_start(self.g_midiplayer_radio, gtk.FALSE)
        combo = gtk.GtkCombo()
        combo.set_popdown_strings(('drvmidi -i d %s',
                                   'timidity -idqq %s',
                                   'playmidi -f %s'))
        self.g_midiplayer_entry = combo.entry
        hbox.pack_start(combo, gtk.FALSE)
        self.g_midiplayer_entry.set_text(self.get_string("sound/commandline"))
        if self.get_string("sound/type") == "external-midiplayer":
            self.g_midiplayer_radio.set_active(gtk.TRUE)
        elif self.get_string("sound/type") == "sequencer-device":
            self.g_device_radio.set_active(gtk.TRUE)
        else:
            self.g_fakesynth_radio.set_active(gtk.TRUE)

        # checkbox to enable awe support
        self.g_awe_checkbutton = gtk.GtkCheckButton(_("My sound card is Sound Blaster AWE32, AWE64 or pnp32"))
        self.g_awe_checkbutton.set_active(self.get_string('sound/card_info') == 'awe')
        if not configureoutput.HAVE_LINUX_AWE_VOICE_H:
            self.g_awe_checkbutton.set_sensitive(gtk.FALSE)
        vbox.pack_start(self.g_awe_checkbutton)

        gu.bButton(vbox, _("Sound test. Hit 'Apply' before testing."), self.on_sound_test)

        self.g_fakesynth_radio.connect('clicked', self.on_changed)
        self.g_midiplayer_radio.connect('clicked', self.on_changed)
        self.g_device_radio.connect('clicked', self.on_changed)
        self.g_midiplayer_entry.connect('changed', self.on_changed)
        self.g_device_entry.connect('changed', self.on_changed)
        self.g_synth_num.connect('changed', self.on_changed)
        self.g_awe_checkbutton.connect('clicked', self.on_changed)
    def create_win32_sound_page(self):
        vbox = gtk.GtkVBox()
        vbox.set_border_width(gnome.uiconsts.PAD)
        self.append_page(vbox, gtk.GtkLabel(_("Sound setup")))
        txt = gtk.GtkLabel(_("""Solfege has two ways to play music. It is recommended to use Windows multimedia output. Support for external midiplayer is only available because this was the only way to get sound on Windows in earlier versions of this program."""))
        txt.set_line_wrap(1)
        txt.set_justify(gtk.JUSTIFY_FILL)
        txt.set_alignment(0.0, 0.0)
        vbox.pack_start(txt, gtk.FALSE)
        self.g_fakesynth_radio = gu.RadioButton(None, _("No sound"), None)
        vbox.pack_start(self.g_fakesynth_radio, gtk.FALSE)
        hbox = gu.bHBox(vbox, gtk.FALSE)
        self.g_device_radio = gu.RadioButton(self.g_fakesynth_radio,
              _("Windows multimedia output, synth number:"), None)
        self.g_synth_num = gtk.GtkSpinButton(gtk.GtkAdjustment(0, 0, 100, 1, 1),
                             digits=0)
        self.g_synth_num.set_value(self.get_int('sound/synth_number'))
        hbox.pack_start(self.g_device_radio, gtk.FALSE)
        hbox.pack_start(self.g_synth_num, gtk.FALSE)

        hbox = gu.bHBox(vbox, gtk.FALSE)
        self.g_midiplayer_radio = gu.RadioButton(self.g_fakesynth_radio,
             _("Use external midiplayer"), None)
        hbox.pack_start(self.g_midiplayer_radio, gtk.FALSE)
        combo = gtk.GtkCombo()
        combo.set_popdown_strings(('midiplayer.exe %s',
                                   'timidity -idqq %s',
                                   'playmidi -f %s'))
        self.g_midiplayer_entry = combo.entry
        hbox.pack_start(combo, gtk.FALSE)
        self.g_midiplayer_entry.set_text(self.get_string("sound/commandline"))
        if self.get_string("sound/type") == "external-midiplayer":
            self.g_midiplayer_radio.set_active(gtk.TRUE)
        elif self.get_string("sound/type") == "winsynth":
            self.g_device_radio.set_active(gtk.TRUE)
        else:
            self.g_fakesynth_radio.set_active(gtk.TRUE)

        gu.bButton(vbox, _("Sound test. Hit 'Apply' before testing."), self.on_sound_test)

        self.g_fakesynth_radio.connect('clicked', self.on_changed)
        self.g_midiplayer_radio.connect('clicked', self.on_changed)
        self.g_synth_num.connect('changed', self.on_changed)
        self.g_device_radio.connect('clicked', self.on_changed)
        self.g_midiplayer_entry.connect('changed', self.on_changed)
    def on_sound_test(self, w):
        mpd.play_music(r"""
        \staff\relative c{
          c16 e g c e, g c e g, c e g c4
        }
        \staff{
          c4 e g8 e c4
        }
        """, 130, 0, 100)
    def on_help(self, *v):
        #FIXME It is stupid to display this help in the main window.
        self.m_app.handle_href("preferences-window.html")
    def on_changed(self, _o=None):
        if self._is_alive:
            self.set_modified(gtk.TRUE)
    def on_apply(self, _o, pagenum):
        card_info = ""
        if pagenum == 0:
            self.set_int('config/slow_bpm', int(self.g_slow_bpm.get_text()))
            self.set_int('config/default_bpm',
                         int(self.g_default_bpm.get_text()))
            self.set_int('config/arpeggio_bpm',
                         int(self.g_arpeggio_bpm.get_text()))
            self.set_int('config/preferred_instrument',
                         self.g_instrsel.m_instrument)
            self.set_int('config/preferred_instrument_velocity',
                         self.g_instrsel.g_velocity.get_value_as_int())
        if pagenum == 1:
            self.set_string('user/lowest_pitch',
                            self.g_lowest_singable.get_value())
            self.set_string('user/highest_pitch',
                            self.g_highest_singable.get_value())
            if self.g_sex_male.active:
                self.set_string('user/sex', 'male')
            else:
                self.set_string('user/sex', 'female')
        if pagenum == 2:
            if self.g_toolbar_text.active:
                self.m_app.m_ui.g_toolbar.set_style(gtk.TOOLBAR_TEXT)
                self.set_string('gui/toolbar_style', 'text')
            elif self.g_toolbar_icons.active:
                self.m_app.m_ui.g_toolbar.set_style(gtk.TOOLBAR_ICONS)
                self.set_string('gui/toolbar_style', 'icons')
            elif self.g_toolbar_both.active:
                self.m_app.m_ui.g_toolbar.set_style(gtk.TOOLBAR_BOTH)
                self.set_string('gui/toolbar_style', 'both')
            if self.g_toolbar_show.active:
                self.m_app.m_ui.get_dock_item_by_name('main-toolbar').show()
                self.set_bool('gui/toolbar_visible', 1)
            else:
                self.set_bool('gui/toolbar_visible', 0)
                self.m_app.m_ui.get_dock_item_by_name('main-toolbar').hide()

            if self.g_navbar_text.active:
                self.m_app.m_ui.g_navbar.set_style(gtk.TOOLBAR_TEXT)
                self.set_string('gui/navbar_style', 'text')
            elif self.g_navbar_icons.active:
                self.m_app.m_ui.g_navbar.set_style(gtk.TOOLBAR_ICONS)
                self.set_string('gui/navbar_style', 'icons')
            elif self.g_navbar_both.active:
                self.m_app.m_ui.g_navbar.set_style(gtk.TOOLBAR_BOTH)
                self.set_string('gui/navbar_style', 'both')
            if self.g_navbar_show.active:
                self.m_app.m_ui.get_dock_item_by_name('backandforward').show()
                self.set_bool('gui/navbar_visible', 1)
            else:
                self.m_app.m_ui.get_dock_item_by_name('backandforward').hide()
                self.set_bool('gui/navbar_visible', 0)
            self.set_bool('gui/mainwin_user_resizeable',
                          self.g_mainwin_user_resizeable.active)
            if self.get_bool('gui/mainwin_user_resizeable'):
                self.m_app.m_ui.set_policy(0, 1, 0)
            else:
                self.m_app.m_ui.set_policy(0, 0, 1)
            self.set_string("config/web_browser", self.g_web_browser.get_text())
            self.set_string("config/mua", self.g_mail_program.get_text())
            self.set_bool('gui/development_menu_visible',
                          self.g_development_menu_visible.active)
            if self.get_bool('gui/development_menu_visible'):
                self.m_app.m_ui.get_dock_item_by_name("Menubar").children()[0].children()[1].show()
            else:
                self.m_app.m_ui.get_dock_item_by_name("Menubar").children()[0].children()[1].hide()
        if pagenum == 3:
            self.set_bool('config/picky_on_new_question',
                          self.g_picky_on_new_question.active)
            self.set_bool('config/auto_repeat_question_if_wrong_answer',
                          self.g_autorepeat_if_wrong.active)
        if pagenum == 4:
            if soundcard.synth:
                soundcard.synth.close()
            if self.g_midiplayer_radio.active:
                soundcard.initialise_external_midiplayer(
                      self.g_midiplayer_entry.get_text())
            elif self.g_device_radio.active:
                if sys.platform != 'win32' and configureoutput.HAVE_LINUX_AWE_VOICE_H and \
                    self.g_awe_checkbutton.active:
                      card_info = "awe"
                else:
                     card_info = ""
                try:
                    if sys.platform == 'win32':
                        soundcard.initialise_winsynth(self.g_synth_num.get_value_as_int())
                    else:
                        soundcard.initialise_devicefile(
                            self.g_device_entry.get_text(),
                            self.g_synth_num.get_value_as_int(),
                            card_info)
                except (soundcard.SoundInitException, OSError), e:
                    self.m_app.display_sound_init_error_message(e)
		    return
            elif self.g_fakesynth_radio.active:
                soundcard.initialise_using_fake_synth(0)
            else:
                raise "Shit"
            if self.g_midiplayer_radio.active:
                self.set_string("sound/type", "external-midiplayer")
            elif self.g_device_radio.active:
                self.set_string("sound/type", "sequencer-device")
            else:
                assert self.g_fakesynth_radio.active
                self.set_string("sound/type", "fake-synth")
            if sys.platform != 'win32':
                self.set_string("sound/device_file", self.g_device_entry.get_text())
                self.set_string("sound/card_info", card_info)
                if soundcard.synth.m_type_major == "OSS":
                    self.set_int("sound/synth_number", soundcard.synth.m_devnum)
                    # we set the spin just in case m_devnum was changed by the
                    # soundcard setup code, if it was out of range
                    self.g_synth_num.set_value(soundcard.synth.m_devnum)
            self.set_string("sound/commandline",
                            self.g_midiplayer_entry.get_text())
        if pagenum == -1:
            pass





# GNU Solfege - eartraining for GNOME
# Copyright (C) 2000, 2001, 2002  Tom Cato Amundsen
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

import gtk
import gnome.uiconsts
import soundcard
from i18n import _
import cfg

#  Prefixes used in this module:
#  t  pack into a table, the first five parameters are table, x1, x2, y1, y2
#  n  a widget that get its state stored in ~/.gnome/Solfege
#  b  the widget is packed into the first argument


def tLabel(table, x1, x2, y1, y2, text="", xalign=0.0, yalign=0.5, xoptions=gtk.EXPAND|gtk.FILL, yoptions=gtk.EXPAND|gtk.FILL, xpadding=0, ypadding=0):
    label = gtk.GtkLabel(text)
    label.set_alignment(xalign, yalign)
    table.attach(label, x1, x2, y1, y2, xoptions=xoptions, yoptions=yoptions, xpadding=xpadding, ypadding=ypadding)
    return label

def bLabel(pack_into, label, expand=gtk.TRUE, fill=gtk.TRUE):
    b = gtk.GtkLabel(label)
    b.show()
    pack_into.pack_start(b, expand, fill)
    return b

def bButton(pack_into, label, callback=None, expand=gtk.TRUE, fill=gtk.TRUE):
    b = gtk.GtkButton(label)
    b.show()
    if callback:
        b.connect('clicked', callback)
    pack_into.pack_start(b, expand, fill)
    return b

class nSpinButton(gtk.GtkSpinButton, cfg.ConfigUtils):#FIXME (??what is there to fix???)
    def __init__(self, exname, name, adj, climb_rate=1, digits=1):
        gtk.GtkSpinButton.__init__(self, adj, climb_rate, digits)
        cfg.ConfigUtils.__init__(self, exname)
        self.m_name = name
        self.set_digits(0)
        self.show()
        self.set_value(self.get_float(self.m_name))
        if self.get_value() != self.get_float(self.m_name):
            self.set_float(self.m_name, self.get_value())
        self.connect('changed', self.on_changed)
        self._watch_id = self.add_watch(self.m_name, self._watch_cb)
    def _watch_cb(self, name):
        gtk.GtkSpinButton.set_value(self, self.get_float(name))
    def set_value(self, value):
        gtk.GtkSpinButton.set_value(self, value)
        self.set_float(self.m_name, value)
    def on_changed(self, _o):
        self.set_float(self.m_name, self.get_value())

def tSpinButton(table, x1, x2, y1, y2,
                value, lower, upper, step_incr=1, page_incr=10, callback=None):
    adj = gtk.GtkAdjustment(value, lower, upper, step_incr, page_incr)
    spin = gtk.GtkSpinButton(adj, digits=0)
    if callback:
        spin.connect('changed', callback)
    table.attach(spin, x1, x2, y1, y2)
    return spin

def bHBox(pack_into, expand=gtk.TRUE, fill=gtk.TRUE, padding=0):
    b = gtk.GtkHBox()
    b.show()
    pack_into.pack_start(b, expand, fill, padding)
    return b

def bVBox(pack_into, expand=gtk.TRUE, fill=gtk.TRUE, padding=0):
    b = gtk.GtkVBox()
    pack_into.pack_start(b, expand, fill, padding)
    return b

class nCheckButton(gtk.GtkCheckButton, cfg.ConfigUtils):
    def __init__(self, exname, name, label=None, default_value=0, callback=None):
        gtk.GtkCheckButton.__init__(self, label)
        #cfg.ConfigUtils.__init__(self, exname)
        cfg.ConfigUtils.__dict__['__init__'](self, exname)
        self.m_name = name
        self.m_callback = callback
        self.show()
        if default_value:
            s = "true"
        else:
            s = "false"
        self.set_bool(self.m_name, self.get_bool(self.m_name+"="+s))
        if self.get_bool(self.m_name):
            self.set_active(1)
        self._clicked_id = self.connect('clicked', self.on_clicked)
        self._watch_id = self.add_watch(self.m_name, self._watch_cb)
    def _watch_cb(self, name):
        self.set_active(self.get_bool(name))
    def on_clicked(self, _o):
        self.set_bool(self.m_name, self.get_active())
        if self.m_callback:
            self.m_callback(_o)
 
def RadioButton(group, label, callback=None):
    rdb = gtk.GtkRadioButton(group, label)
    if callback:
        rdb.connect('toggled', callback)
    rdb.show()
    return rdb

class nCombo(gtk.GtkCombo, cfg.ConfigUtils):
    def __init__(self, exname, name, default, popdown_strings):
        """
        Be aware that the value of the entry, is stored as an integer
        popdown_strings.index(entry.get_text()), so if popdown_strings
        changes when upgrading the program, the value of the combo
        might change.

        Despite this problems, I do it this way, because if we store
        the actual value of the entry, we get into trouble when running
        the program with other locale settings.
        """
        gtk.GtkCombo.__init__(self)
        #cfg.ConfigUtils.__init__(self, exname)
        cfg.ConfigUtils.__dict__['__init__'](self, exname)
        self.popdown_strings = popdown_strings
        self.m_name = name
        self.set_value_in_list(gtk.TRUE, gtk.FALSE)
        self.set_popdown_strings(popdown_strings)
        i = self.get_int_with_default(name, -1)
        if i == -1:
            i = popdown_strings.index(default)
        self.entry.set_text(popdown_strings[i])
        self.entry.connect("changed", self.entry_changed)
        self.entry.set_editable(gtk.FALSE)
        self.show()
    def entry_changed(self, entry):
        self.set_int(self.m_name, self.popdown_strings.index(entry.get_text()))

class PercussionNameCombo(gtk.GtkCombo, cfg.ConfigUtils):
    def __init__(self, exname, name, default):
        gtk.GtkCombo.__init__(self)
        cfg.ConfigUtils.__init__(self, exname)
        self.m_name = name
        self.set_value_in_list(gtk.TRUE, gtk.FALSE)
        self.set_popdown_strings(soundcard.percussion_names)
        i = self.get_int(name)
        if not i:
            i = soundcard.percussionname_to_int(default)
            self.set_int(name, i)
        self.entry.set_text(soundcard.int_to_percussionname(i))
        self.entry.connect("changed", self.entry_changed)
        self.entry.set_editable(gtk.FALSE)
        self.show()
    def entry_changed(self, entry):
        self.set_int(self.m_name,
                     soundcard.percussionname_to_int(entry.get_text()))

class SolfegeDialog(gtk.GtkDialog):
    def __init__(self, heading, txt, buttons=(), callback=None):
        gtk.GtkDialog.__init__(self)
        self.m_callback = callback
        self.vbox.set_border_width(gnome.uiconsts.PAD)
        self.connect('delete_event', self.destroy)

        label = bLabel(self.vbox, heading)
        label.set_name('Heading1')

        label = bLabel(self.vbox, txt)
        label.set_justify(gtk.JUSTIFY_LEFT)
        for text in buttons:
            b = gtk.GtkButton(text)
            b.set_data('user_data', text)
            b.connect('clicked', self.click)
            self.action_area.pack_start(b)
            b.show()
    def click(self, button):
        if self.m_callback:
            self.m_callback(button.get_data('user_data'))
        self.destroy()


class FlashBar(gtk.GtkFrame):
    def __init__(self):
        gtk.GtkFrame.__init__(self)
        self.set_shadow_type(gtk.SHADOW_IN)
        #UGH
        self.set_usize(358, 40)
        self.__stack = []
        import widgets
        self.__label = widgets.HarmonicProgressionLabel('')
        self.add(self.__label)
        self.__timeout = None
    def flash(self, txt):
        """Display a message that is automatically removed after some time.
        If we flash a new message before the old flashed message are removed,
        we old flashed message are removed.
        """
        if self.__timeout:
            gtk.timeout_remove(self.__timeout)
        self.__label.set_text(txt)
        def f(self=self):
            self.__timeout = None
            if self.__stack:
                self.__label.set_text(self.__stack[-1])
            else:
                self.__label.set_text('')
        self.__timeout = gtk.timeout_add(2000, f)
    def push(self, txt):
        # stop any flashing before we push
        if self.__timeout:
            gtk.timeout_remove(self.__timeout)
            self.__timeout = None
        self.__stack.append(txt)
        self.__label.set_text(txt, '')
    def pop(self):
        """If a message is being flashed right now, that flashing is
        not affected, but the message below the flashed message is removed.
        """
        self.__stack.pop()
        if not self.__timeout:
            if self.__stack:
                self.__label.set_text(self.__stack[-1])
    def clear(self):
        self.__stack = []
        if not self.__timeout:
            self.__label.set_text('')



#!/usr/bin/python2.1
import sys, os
sys.path.append("/usr/lib/python1.5/site-packages")
if os.path.isdir("soundcard"):
    sys.path.append(".")
else:
    sys.path.append("..")

import gnome.ui
import mpd, mpd.musicdisplayer, soundcard
import gtk
from src import gu, cfg
cfg.initialise(None, None, 'mpd-test')
import src.utils

class DisplaytestWindow(gtk.GtkWindow):
    def on_quit(self, w):
        cfg.sync()
    def __init__(self):
        gtk.GtkWindow.__init__(self)
        self.connect('destroy', self.on_quit)
        self.vbox = vbox = gtk.GtkVBox()
        vbox.show()
        self.add(vbox)
        self.g_text = gtk.GtkText()
        self.g_text.show()
        self.g_text.set_editable(gtk.TRUE)
        s = cfg.get_string('config/score_displayer')
        if not s:
            s = r"\staff{c' d' e'}"
        self.g_text.insert_defaults(s)
        vbox.pack_start(self.g_text)
        self.g_displayer = mpd.musicdisplayer.MusicDisplayer(src.utils.play_tone)
        self.g_displayer.set_usize(200, 200)
        self.g_displayer.show()
        self.vbox.pack_start(self.g_displayer)
        gu.bButton(vbox, "Parse", self.on_parse)
        gu.bButton(vbox, "Display", self.on_display)
        gu.bButton(vbox, "Display first notes", self.on_display_first_notes)
        #gu.bButton(vbox, "Display last notes", self.on_display_last_notes)
        gu.bButton(vbox, "Play", self.on_play)
        gu.bButton(vbox, "Play first", self.on_play_first)
    def on_parse(self, _o):
        t = self.g_text.get_chars(0, -1)
        score = mpd.parser.parse_to_score_object(t)
    def on_display(self, _o):
        t = self.g_text.get_chars(0, -1)
        cfg.set_string('config/score_displayer', t)
        self.g_displayer.display(t, 20)
    def on_display_first_notes(self, _o):
        t = self.g_text.get_chars(0, -1)
        cfg.set_string('config/score_displayer', t)
        self.g_displayer.display(t, 20, mpd.FIRST)
    def on_display_last_notes(self, _o):
        t = self.g_text.get_chars(0, -1)
        cfg.set_string('config/score_displayer', t)
        self.g_displayer.display(t, 20, mpd.LAST)
    def on_play(self, _o):
        t = self.g_text.get_chars(0, -1)
        cfg.set_string('config/score_displayer', t)
        mpd.play_music(t, (120, 4), 0, 100)
    def on_play_first(self, _o):
        t = self.g_text.get_chars(0, -1)
        tr = mpd.music_to_track(t, 100, soundcard.rat.Rat(0, 1), soundcard.rat.Rat(1, 8))
        soundcard.synth.play_track(tr)


#soundcard.initialise_devicefile("/dev/sequencer", 0)
soundcard.initialise_devicefile("/dev/music", 0)
#soundcard.initialise_using_fake_synth()

w = DisplaytestWindow()
w.connect('destroy', gtk.mainquit)
w.show()
gtk.mainloop()



# GNU Solfege - eartraining for GNOME
# Copyright (C) 2000, 2001, 2002  Tom Cato Amundsen
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

import gtk, gnome
import abstract, idbyname, const, gu, statisticsviewer, widgets
from i18n import _
import soundcard, mpd, mpd.musicdisplayer
import utils

class Teacher(idbyname.Teacher):
    def __init__(self, exname, app):
        idbyname.Teacher.__init__(self, exname, app, const.USE_HARMONY)
    def play_question(self):
        if self.q_status == const.QSTATUS_NO:
            return
        tracklist = mpd.music_to_tracklist(self.m_P.get_music(),
                 self.get_int('config/preferred_instrument_velocity'))
        l, lv, m, mv, h, hv = self.get_instrument_config(3)
        tracklist[0].prepend_patch(0, h)
        t = tracklist[0]
        for x in range(1, len(tracklist)-1):
            tracklist[x].prepend_patch(x, m)
            t.merge_with(tracklist[x])
        tracklist[-1].prepend_patch(len(tracklist)-1, l)
        t.merge_with(tracklist[-1])
        soundcard.synth.play_track(t)

class Gui(abstract.LessonbasedGui):
    def __init__(self, teacher, window):
        abstract.LessonbasedGui.__init__(self, teacher, window)
        self.m_key_bindings = {'new_ak': self.new_question,
                               'repeat_ak': self.m_t.play_question,
                               'guess_answer_ak': self.guess_answer,
                               'show_ak':  self.show_answer}
        ################
        # practise_box #
        ################
        self.g_music_displayer = mpd.musicdisplayer.MusicDisplayer(utils.play_tone)
        self.g_music_displayer.set_usize(-1, 170)
        self.practise_box.pack_start(self.g_music_displayer)

        self.g_flashbar = gu.FlashBar()
        self.g_flashbar.show()
        self.practise_box.pack_start(self.g_flashbar, gtk.FALSE)
        self.practise_box.set_spacing(gnome.uiconsts.PAD)
 
        self.g_entry = gtk.GtkEntry()
        self.practise_box.pack_start(self.g_entry, gtk.FALSE)
        self.g_new = gu.bButton(self.action_area, _("New"), self.new_question)
        self.g_repeat = gu.bButton(self.action_area, _("Repeat"),
                                   lambda _o, self=self: self.m_t.play_question())
        self.g_repeat.set_sensitive(gtk.FALSE)
        self.g_guess_answer = gu.bButton(self.action_area, _("Guess answer"), 
                                         self.guess_answer)
        self.g_guess_answer.set_sensitive(gtk.FALSE)
        self.g_show = gu.bButton(self.action_area, _("Show"), self.show_answer)
        self.g_show.set_sensitive(gtk.FALSE)
        self.practise_box.show_all()
        ##############
        # config_box #
        ##############
        self.g_lessonfile_selector = widgets.SelectLessonfileWidget(
           self.m_t, self.g_win, self.update_gui_after_lessonfile_change)
        self.g_lessonfile_selector.show()
        self.config_box.pack_start(self.g_lessonfile_selector)
        #self.g_instrument_configurator \
        #    = widgets.InstrumentConfigurator(self.m_exname, 3)
        #self.config_box.pack_start(self.g_instrument_configurator, gtk.FALSE)
        #self.g_instrument_configurator.show()

        ###############
        # statistics
        ###############
        self.setup_statisticsviewer(statisticsviewer.StatisticsViewer,
                                   _("Harmonic progression dictation"))

        self.update_gui_after_lessonfile_change()
    def guess_answer(self, widget=None):
        if self.m_t.q_status == const.QSTATUS_NO:
            return
        if self.m_t.q_status == const.QSTATUS_SOLVED:
            if self.m_t.guess_answer(self.g_entry.get_text()):
                self.g_flashbar.flash(_("Correct, but you have already solved this question"))
            else:
                self.g_flashbar.flash(_("Wrong, but you have already solved this question"))
        else:
            if self.m_t.guess_answer(self.g_entry.get_text()):
                self.g_flashbar.flash(_("Correct"))
                self.g_new.set_sensitive(gtk.TRUE)
            else:
                self.g_flashbar.flash(_("Wrong"))
    def show_answer(self, widget=None):
        if self.m_t.q_status != const.QSTATUS_NO:
            self.g_music_displayer.display(self.m_t.m_P.get_music(),
                               self.get_int('config/feta_font_size=20'))
    def new_question(self, widget=None):
        if self.m_t.new_question() == self.m_t.OK:
            self.g_music_displayer.display(self.m_t.m_P.get_music(),
                        self.get_int('config/feta_font_size=20'), mpd.FIRST)
            self.g_repeat.set_sensitive(gtk.TRUE)
            self.g_show.set_sensitive(gtk.TRUE)
            self.g_guess_answer.set_sensitive(gtk.TRUE)
            self.g_new.set_sensitive(
                 not self.get_bool('config/picky_on_new_question'))
            self.m_t.play_question()
    def on_end_practise(self):
        self.g_new.set_sensitive(gtk.TRUE)
        self.g_repeat.set_sensitive(gtk.FALSE)
        self.g_guess_answer.set_sensitive(gtk.FALSE)
        self.g_show.set_sensitive(gtk.FALSE)
        self.g_music_displayer.clear()
        self.m_t.end_practise()
    def update_gui_after_lessonfile_change(self):
        self.g_music_displayer.clear()
        if self.m_t.m_P:
            self.g_new.set_sensitive(gtk.TRUE)
        else:
            self.g_new.set_sensitive(gtk.FALSE)
        self.g_repeat.set_sensitive(gtk.FALSE)
        self.g_guess_answer.set_sensitive(gtk.FALSE)
        self.g_show.set_sensitive(gtk.FALSE)
        self.m_t.m_statistics.lessonfile_changed(
                     self.get_string('lessoncollection'),
                     self.get_string('lessonfile'))
        self.g_statview.m_descriptive_paragraph = \
         _("Statistics for the lesson file '%s'") % self.get_string('lessonfile')
        self.g_win.set_title("Solfege - " + self.get_pretty_name())



# GNU Solfege - eartraining for GNOME
# Copyright (C) 2000, 2001, 2002  Tom Cato Amundsen
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

import gtk, gnome, gu
import const
from i18n import _
import cfg

class IntervallCheckBox(gtk.GtkHBox, cfg.ConfigUtils):
    def __init__(self, exname, varname):
        gtk.GtkHBox.__init__(self)
        cfg.ConfigUtils.__init__(self, exname)
        self.m_varname = varname
        intervalls = self.get_list(self.m_varname)
        if not intervalls:
            intervalls = range(1, 17)
            self.set_list(varname, intervalls)
        self.checkbox_dict = {}
        for x in range(1, 17):
            self.checkbox_dict[x] = c = gtk.GtkToggleButton(const.short_intervall_name[x])
            c.set_data('intervall', x)
            if x in intervalls:
                c.set_active(gtk.TRUE)
            c.connect('clicked', self.on_toggle)
            c.show()
            self.pack_start(c)
        self.add_watch(self.m_varname, self._watch_cb)
    def _watch_cb(self, name):
        V = self.get_list(self.m_varname)
        for k in range(1, 17):
            self.checkbox_dict[k].set_active(k in V)
    def on_toggle(self, _o):
        v = []
        keys = self.checkbox_dict.keys()
        for k in keys:
            if self.checkbox_dict[k].active:
                v.append(k)
        self.set_list(self.m_varname, v)


class MultipleIntervallConfigWidget(gtk.GtkVBox, cfg.ConfigUtils):
    """
    This class gives  you some spin buttons and two rows of buttons
    where you select what intervalls to use for exercises where you
    select one or more intervalls.
    """
    def __init__(self, exname):
        gtk.GtkVBox.__init__(self)
        cfg.ConfigUtils.__init__(self, exname)
        self.MAX_INT = self.get_int('maximum_number_of_intervalls')
        if self.MAX_INT == 0:
            self.MAX_INT = 10
        self._watched_intervall_id = None
        self._watched_intervall = None
        self.m_ignore_iclick = 0
        box = gu.bHBox(self, expand=gtk.FALSE)
        box.set_spacing(gnome.uiconsts.PAD_SMALL)
        box.pack_start(gtk.GtkLabel(_("Number of intervalls:")), gtk.FALSE)
        self.g_num_int_spin = gu.nSpinButton(self.m_exname,
                       'number_of_intervalls',
                       gtk.GtkAdjustment(1, 1, self.MAX_INT, 1, 10))
        self.add_watch('number_of_intervalls', self.on_num_int_spin)
        box.pack_start(self.g_num_int_spin, gtk.FALSE)
        self.g_all_int_button = gu.bButton(box,
                  _("Configure all intervalls in this exercise like this"),
                  self.configure_all_like_active_intervall, expand=gtk.FALSE)
        self.add_watch('number_of_intervalls', lambda n, self=self: \
               self.g_all_int_button.set_sensitive(self.get_int(n)!=1))

        box = gu.bHBox(self, expand=gtk.FALSE)
        box.set_spacing(gnome.uiconsts.PAD_SMALL)
        box.pack_start(gtk.GtkLabel(
                _("Togglebuttons is for intervall number:")), gtk.FALSE)
        self.m_int_sel_adjustment \
             = gtk.GtkAdjustment(1, 1, self.get_int('number_of_intervalls'), 1)
        self.g_int_sel_spin = gu.nSpinButton(self.m_exname,
                  'cur_edit_intervall', self.m_int_sel_adjustment, digits=0)
        self.g_int_sel_spin.connect('changed', self.on_int_sel_spin)
        box.pack_start(self.g_int_sel_spin, gtk.FALSE)
        table = gtk.GtkTable()
        self.pack_start(table, gtk.FALSE)

        label = gtk.GtkLabel(_("Up:"))
        label.set_alignment(1.0, 0.5)
        table.attach(label, 0, 1, 0, 1, xpadding=gnome.uiconsts.PAD_SMALL)
        self.g_intervall_chk = {}
        V = self.get_list("ask_for_intervalls_%i"
                  % (self.g_int_sel_spin.get_value_as_int()-1))
        for x in range(1, 17):
            self.g_intervall_chk[x] = c = gtk.GtkToggleButton(const.short_intervall_name[x])
            c.set_data('intervall', x)
            if x in V:
                c.set_active(gtk.TRUE)
            c.show()
            c.connect('clicked', self.on_intervall_chk_clicked)
            table.attach(c, x, x+1, 0, 1)

        box = gu.bHBox(self, expand=gtk.FALSE)
        label = gtk.GtkLabel(_("Down:"))
        label.set_alignment(1.0, 0.5)
        table.attach(label, 0, 1, 1, 2, xpadding=gnome.uiconsts.PAD_SMALL)
        v = range(-16, 0)
        v.reverse()
        for x in v:
            self.g_intervall_chk[x] = c = gtk.GtkToggleButton(const.short_intervall_name[-x])
            c.set_data('intervall', x)
            if x in V:
                c.set_active(gtk.TRUE)
            c.show()
            c.connect('clicked', self.on_intervall_chk_clicked)
            table.attach(c, -x, -x+1, 1, 2)
        table.show_all()
        if self.g_num_int_spin.get_value_as_int() == 1:
            self.g_all_int_button.set_sensitive(gtk.FALSE)
        gu.bButton(self, _("Reset to default values"), self.reset_to_default,
                  gtk.FALSE, gtk.FALSE)
        self._watch_intervall(self.get_int('cur_edit_intervall')-1)
    def reset_to_default(self, _o):
        self.set_int('cur_edit_intervall', 1)
        self.set_int('number_of_intervalls', 1)
        self.set_list('ask_for_intervalls_0', range(-16, 0)+range(1, 17))
    def configure_all_like_active_intervall(self, _o):
        v = self.get_list('ask_for_intervalls_%i' \
                               % (self.g_int_sel_spin.get_value_as_int()-1))
        for i in range(self.get_int('number_of_intervalls')):
            self.set_string('ask_for_intervalls_%i' % i, str(v))
    def on_intervall_chk_clicked(self, _o):
        if self.m_ignore_iclick:
            return
        i = self.g_int_sel_spin.get_value_as_int() - 1
        v = self.get_list('ask_for_intervalls_%i' % i)
        if _o.active:
            if not _o.get_data('intervall') in v:
                v.append(_o.get_data('intervall'))
        else:
            if _o.get_data('intervall') in v:
                del v[v.index(_o.get_data('intervall'))]
        self.set_list('ask_for_intervalls_%i' % i, v)
    def on_num_int_spin(self, _o):
        a = self.g_int_sel_spin.get_adjustment()
        #a.upper = self.get_int('number_of_intervalls')
        a.set_all(self.get_int('cur_edit_intervall'), 1,
                  self.get_int('number_of_intervalls'), 1, 10, 10)
        self.g_int_sel_spin.set_adjustment(a)
        self.g_int_sel_spin.update()
    def on_int_sel_spin(self, _o):
        self._watch_intervall(self.get_int('cur_edit_intervall')-1)
        self.update_togglebuttons(self.get_int('cur_edit_intervall')-1)
    def update_togglebuttons(self, i):
        self.m_ignore_iclick = 1
        v = self.get_list('ask_for_intervalls_%i' % i)
        for i in range(-16, 0)+range(1, 17):
            self.g_intervall_chk[i].set_active(i in v)
        self.m_ignore_iclick = 0
    def _watch_intervall(self, i):
        """
        Add a watch for the correct ask_for_intervalls_%i variable.
        Remove old watches. Do nothing if we are already watching the
        correct data.
        """
        if self._watched_intervall == i:
            return
        if self._watched_intervall != None:
            self.remove_watch('ask_for_intervalls_%i' % self._watched_intervall,
                          self._watched_intervall_id)
        self._watched_intervall = i
        self._watched_intervall_id = self.add_watch('ask_for_intervalls_%i' % i,
                                                  self._intervall_watch_cb)
    def _intervall_watch_cb(self, name):
        self.update_togglebuttons(self._watched_intervall)



#!/usr/bin/env python
import sys, os
if os.getcwd()[-4:] == "test":
    sys.path.insert(0, "..")
else:
    sys.path.insert(0, ".")
import soundcard
import sys
soundcard.initialise("/dev/sequencer", 0)
#soundcard.initialise("/hom/tom/src/playmidi-2.4/playmidi -a %s")


def f():
        t = soundcard.Track()
        t.start_note(1, 50, 100);
        t.notelen_time(4)
        t.stop_note(1, 50, 100);
        t.start_note(1, 51, 100);
        t.notelen_time(4)
        t.stop_note(1, 51, 100);
        return t
def f2():
        t = soundcard.Track()
        t.set_patch(0, 66)
        t.set_patch(1, 66)
        t.start_note(0, 60, 100)
        t.notelen_time(4)
        t.start_note(0, 60, 100)
        t.notelen_time(4)
        t.stop_note(0, 60, 100)
        t.notelen_time(4)
        t.stop_note(0, 60, 100)
        return t
t = f2()
print "starter og spille"
soundcard.synth.play_track(t)
print "press ENTER to continue"
sys.stdin.readline()
soundcard.synth.close()



# GNU Solfege - eartraining for GNOME
# Copyright (C) 2000, 2001, 2002  Tom Cato Amundsen
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

from instrumentselector import InstrumentSelector, InstrumentConfigurator
from notenamespinbutton import NotenameRangeController, \
                               nNotenameRangeController, \
                               SimplestNotenameSpinButton
from specialwidgets import QuestionNameButtonTable, HarmonicProgressionLabel
from multipleintervallconfigwidget import MultipleIntervallConfigWidget, \
                                          IntervallCheckBox
from selectlessonfilewidget import SelectLessonfileWidget



#!/usr/bin/python

# GNU Solfege - eartraining for GNOME
# Copyright (C) 2000, 2001, 2002  Tom Cato Amundsen
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

import os, sys, string

RM = "tmp/tmpsolfege"
prefix = '/usr'

sys.stdout.write("%defattr(-,root,root)\n")
infile = sys.stdin
while 1:
    s = infile.readline()
    if not s:
        break
    s = '/'+s[:-1][len(RM)+2:]
    if s: #UGH WHY WHY WHY!?!
        if s == '/usr/man/man1/solfege.1':
            s = s + "*"
        sys.stdout.write(s)
        sys.stdout.write("\n")




#!/usr/bin/env python

import profile, os
import types, sys

sys.path.insert(0, "gnomeemu")
sys.path.append(".")
sys.path.append("/usr/lib/python1.5/site-packages")

import src.lessonfile

def par():
	for n in range(100):
		src.lessonfile.DictationLessonfile(sys.argv[1])

import time
print "Startet tidtaking:"
t1 = time.clock()
par()
t2 = time.clock()
print t2-t1
#profile.run("par()", "profile.txt")

#import pstats
#p = pstats.Stats("profile.txt")
##p.strip_dirs().sort_stats('time').print_stats(30)
#p.strip_dirs().sort_stats('cumulative').print_stats(30)

#bas()



# GNU Solfege - eartraining for GNOME
# Copyright (C) 2000, 2001, 2002  Tom Cato Amundsen
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

import __main__

from i18n import _

import configureoutput
import errno, sys
import shutil

import soundcard, mpd
import lessonfile
import os
import cfg, utils

class SolfegeApp(cfg.ConfigUtils):
    def __init__(self, argv, ui):
        cfg.ConfigUtils.__init__(self, 'solfege-app')
        self.m_argv = argv
        self.m_ui = ui
        self.check_rcfile()
        self.check_statistics()
        self.m_teachers = {}
        self.lessonfile_manager = lessonfile.LessonFileManager()
        self.setup_sound()
    def check_rcfile(self):
        # The [app] version variable say  the oldest rc version
        # this function can convert to a version this version of
        # Solfege can handle.
        #
        # First figure out what file to replace
        if sys.platform == "win32":
            fn = "~/solfegerc%s.%s"
        else:
            fn = "~/.solfegerc%s.%s"
        fn = fn % (configureoutput.MAJOR_VERSION,
                   configureoutput.MINOR_VERSION)
        if sys.platform == "win32":
            # FIXME this version number has to be updated in sync with
            # Makefile.in
            ver = "1.1.3"
        else:
            ver = "0.7.34"
        if utils.compare_version_strings(
                 cfg.get_string("app/version"), ver) == -1:
            if os.path.isfile("debian-solfege.py.in"):
                # we are running the program from the source dir
                # without installing. Also windows users go here.
                shutil.copyfile("default.config", os.path.expanduser(fn))
                #FIXME use cfg.*
                cfg.reread_data()
            else:
                cfg.drop_user_config()
        if __main__.rc_copied:
            # some checks if we have upgraded an old rcfile from 1.0.x
            # or 1.1.x 
            if cfg.get_string("id-tone/new_ak") == "":
                cfg.set_string("id-tone/new_ak", "<ctrl>n")
            if cfg.get_string("id-tone/give_up_ak") == "":
                cfg.set_string("id-tone/give_up_ak", "<ctrl>g")
            cfg.set_bool("gui/development_menu_visible", 0)
        cfg.set_string('app/version', configureoutput.VERSION_STRING)
    def check_statistics(self):
        # the devel series are fresh and no history to care about
        return
        if cfg.get_string("app/version") < "0.7.21":
            print "Deleting all your statistics because of compatibility issues."
            if os.path.exists(os.path.expanduser("~/.solfege/")):
                shutil.rmtree(os.path.expanduser("~/.solfege/"))
        elif cfg.get_string("app/version") < "0.7.33":
            print "Deleting the identify-scale statistics because there was one"
            print "of the keys to the statistics code was changed. Such nasty"
            print "changes will not happen in the stable 1.0.x series."
            if os.path.exists(os.path.expanduser("~/.solfege0.7/identify-scale")):
                shutil.rmtree(os.path.expanduser("~/.solfege0.7/identify-scale/"))
    def setup_sound(self):
        if sys.platform == 'win32' and \
                    cfg.get_string("sound/type") == "sequencer-device":
            # just in case c:\home\solfegercX.Y is wrong
            cfg.set_string("sound/type", "winsynth")
        if '--no-sound' in self.m_argv \
           or cfg.get_string("sound/type") == "fake-synth":
            soundcard.initialise_using_fake_synth('--verbose-sound-init' in self.m_argv)
        elif cfg.get_string("sound/type") == "winsynth":
            soundcard.initialise_winsynth(cfg.get_int("sound/synth_number"),
                      verbose_init='--verbose-sound-init' in self.m_argv)
        elif cfg.get_string("sound/type") == "external-midiplayer":
            soundcard.initialise_external_midiplayer(
                    cfg.get_string("sound/commandline"),
                    verbose_init='--verbose-sound-init' in self.m_argv)
        elif cfg.get_string("sound/type") == '':
            self.m_ui.display_error_message(
"""You should configure sound
from the 'Sound' page of
the preferences window.
""")
        elif cfg.get_string("sound/type") == "sequencer-device":
            if not configureoutput.HAVE_LINUX_AWE_VOICE_H and \
               cfg.get_string("sound/card_info") == "awe":
                       cfg.set_string("sound/card_info", "");
            try:
                soundcard.initialise_devicefile(
                             cfg.get_string("sound/device_file"),
                             cfg.get_int("sound/synth_number"),
                             cfg.get_string("sound/card_info"),
                             verbose_init='--verbose-sound-init' in self.m_argv)
            except (soundcard.SoundInitException, OSError), e:
                if e == None:
                    self.m_ui.display_error_message(
"""You should configure sound
from the 'Sound' page of
the preferences window.
""")
                else:
                    self.display_sound_init_error_message(e)
    def display_sound_init_error_message(self, e):
        if isinstance(e, soundcard.SoundInitException):
            self.m_ui.display_error_message(
            """%s""" % e)
        elif e.errno == errno.EACCES:
            self.m_ui.display_error_message(
"""The sound init failed: %s
The errno EACCES indicates that you don't have write
permission to the device.""" % e)
        elif e.errno == errno.EBUSY:
            self.m_ui.display_error_message(
"""The sound init failed: %s
It seems like some other program is using the device. You
should try to quit that other program and restart Solfege."""
 % e)
        else:
            self.m_ui.display_error_message(
"""The sound init failed: %s
It seems like your kernel don't have the correct
sound modules installed, or that they are wrongly
configured. You can check the 'Sound' page in the
preferences window, but I don't think anything there
can help you. (This program need better sound init
code to handle situations like this.)"""
 % errno.errorcode[e.errno])
    def play_happy_sound(self):
        mpd.play_music(r"\staff\relative c'{c16 e g a}", 180, 8,
               cfg.get_int('config/preferred_instrument_velocity'))
    def play_sad_sound(self):
        mpd.play_music(r"\staff\relative c'{<c,,8 cis>", 80, 58,
               cfg.get_int('config/preferred_instrument_velocity'))
    def please_help_me(self):
        if self.m_ui.m_viewer != 'docviewer':
            self.handle_href('%s-help.html' % self.m_ui.m_viewer)
    def handle_href(self, href):
        protocol, action, fn, lessoncollection, lessonfile, config = \
           utils.parse_url(href)
        if protocol == None:
            self.m_ui.display_docfile(fn)
        elif protocol == 'solfege':
            if action == 'all-lessonfiles':
                self.m_ui.display_html(self.lessonfile_manager.m_htmldoc)
                self.m_ui.m_history.add('solfege:all-lessonfiles')
            else:
                if not self.m_teachers.has_key(fn):
                    n = utils.exercise_name_to_module_name(fn)
                    exec("import %s" % n)
                    self.m_teachers[fn] = locals()[n].Teacher(fn, self)
                    # if we don't have a teacher, then the Gui has not been
                    # initialized either
                    self.m_ui.initialise_exercise(self.m_teachers[fn])
                    self.m_teachers[fn].g_view = self.m_ui.box_dict[fn]
                self.m_teachers[fn].configure_exercise(lessoncollection, lessonfile, config)
                self.m_ui.activate_exercise(href)
        elif protocol == 'http':
            os.system(self.get_string("config/web_browser") % href)
        elif protocol == 'mailto':
            os.system(self.get_string("config/mua") % href)
        else:
            print "unknown link type", protocol
    def quit_program(self):
        cfg.sync()
        for k in self.m_teachers.keys():
            if self.m_teachers[k].m_statistics is not None:
                self.m_teachers[k].m_statistics.save_data()
        if soundcard.synth:
            soundcard.synth.close()





#!/usr/bin/python

# Copyright is GPL.

# Unfortunately texinfo 4.0f that are in Debian woody are buggy,
# so we have to fix things.

import sys
fn = sys.argv[1]

f = open(fn, 'r')
s = f.readline()
b = ""
while s != "":
    # I know my bad documentation files!
    s = s[:-1]
    for x in range(2, 6):
        if s.startswith('<h%i>' % x) and s.endswith('</h%i>' % (x-1)):
            s = s[:-5] + '</h%i>' % x
    b += s + "\n"
    s = f.readline()
f.close()
f = open(fn, 'w')
f.write(b)
f.close()



# GNU Solfege - eartraining for GNOME
# Copyright (C) 2000, 2001, 2002  Tom Cato Amundsen
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

import gtk, gnome
import gu, widgets
import soundcard, mpd, mpd.musicdisplayer
import abstract, const, lessonfile
import string
import utils
from i18n import _

class Teacher(abstract.LessonbasedTeacher):
    def __init__(self, exname, app):
        abstract.LessonbasedTeacher.__init__(self, exname, app, const.USE_CHORD)
        self.lessonfileclass = lessonfile.ChordLessonfile
        self.parse_lessonfile()
    def new_question(self):
        """
        return 1 if the teacher has a question to ask
        """
        if self.m_P:
            self.m_P.select_random_question()
            self.q_status = const.QSTATUS_NEW
            return 1
    def _generate_question_vector(self):
        if self.get_bool('override_default_instrument'):
            instr_low = self.get_int('lowest_instrument')
            instr_low_vel = self.get_int('lowest_instrument_velocity')
            instr_middle = self.get_int('middle_instrument')
            instr_middle_vel = self.get_int('middle_instrument_velocity')
            instr_high = self.get_int('highest_instrument')
            instr_high_vel = self.get_int('highest_instrument_velocity')
        else:
            instr_low = instr_middle = instr_high \
                        = self.get_int('config/preferred_instrument')
            instr_low_vel = instr_middle_vel = instr_high_vel \
                         = self.get_int('config/preferred_instrument_velocity')
        m = map(string.strip, string.split(self.m_P.get_music()))
        v = []
        for x in m:
            v.append([x, instr_middle, instr_middle_vel])
        v[0][1] = instr_low
        v[0][2] = instr_low_vel
        v[-1][1] = instr_high
        v[-1][2] = instr_high_vel
        return v
    def give_up(self):
        self.m_q_status = const.QSTATUS_SOLVED
    def play_question(self):
        if self.q_status == const.QSTATUS_NO:
            return
        v = self._generate_question_vector()
        m = soundcard.Track()
        m.set_bpm(self.get_int('config/default_bpm'))
        for x in range(len(v)):
            m.set_patch(x, v[x][1])
            m.start_note(x, mpd.notename_to_int(v[x][0]), v[x][2])
        for x in range(len(v)):
            if x == 0:
                m.notelen_time(4)
                m.stop_note(x, mpd.notename_to_int(v[x][0]), v[x][2])
            else:
                m.stop_note(x, mpd.notename_to_int(v[x][0]), v[x][2])
        soundcard.synth.play_track(m)
    def play_question_arpeggio(self):
        if self.q_status == const.QSTATUS_NO:
            return
        v = self._generate_question_vector()
        m = soundcard.Track()
        m.set_bpm(self.get_int('config/arpeggio_bpm'))
        for x in range(len(v)):
            m.set_patch(0, v[x][1])
            m.note(4, 0, mpd.notename_to_int(v[x][0]), v[x][2])
        soundcard.synth.play_track(m)


class Gui(abstract.LessonbasedGui):
    def __init__(self, teacher, window):
        abstract.LessonbasedGui.__init__(self, teacher, window)
        self.m_key_bindings = {'new_ak': self.new_question,
                        'repeat_ak': self.m_t.play_question,
                        'repeat_arpeggio_ak': self.m_t.play_question_arpeggio,
                        'give_up_ak': self.give_up}
        self.m_correct = [0, 0, 0]
        ################
        # practice_box #
        ################
        hbox = gu.bHBox(self.practise_box)
        hbox.set_spacing(gnome.uiconsts.PAD)

        table = gtk.GtkTable(5, 5, 0)
        hbox.pack_start(table, gtk.FALSE)
        hsep = gtk.GtkHSeparator()
        table.attach(hsep, 0, 5, 1, 2, xoptions=gtk.FILL,
                     yoptions=0, ypadding=gnome.uiconsts.PAD_SMALL)
        hsep = gtk.GtkHSeparator()
        table.attach(hsep, 0, 5, 3, 4, xoptions=gtk.FILL,
                     yoptions=0, ypadding=gnome.uiconsts.PAD_SMALL)
        vsep = gtk.GtkVSeparator()
        table.attach(vsep, 1, 2, 0, 5,
                     xoptions=0, xpadding=gnome.uiconsts.PAD_SMALL)
        vsep = gtk.GtkVSeparator()
        table.attach(vsep, 3, 4, 0, 5,
                     xoptions=0, xpadding=gnome.uiconsts.PAD_SMALL)

        table.attach(gtk.GtkLabel(_("Chord type")), 0, 1, 0, 1,
                     xoptions=gtk.FILL, yoptions=0)
        self.g_chordtype_box = gtk.GtkVBox()
        table.attach(self.g_chordtype_box, 0, 1, 2, 3, xoptions=gtk.FILL)
        self.g_type_status = gtk.GtkLabel("")
        table.attach(self.g_type_status, 0, 1, 4, 5,
                     xoptions=gtk.FILL, yoptions=0)

        table.attach(gtk.GtkLabel(_("Inversion")), 2, 3, 0, 1,
                     xoptions=gtk.FILL, yoptions=0)
        self.g_inversion_box = gtk.GtkVBox()
        table.attach(self.g_inversion_box, 2, 3, 2, 3, xoptions=gtk.FILL)
        self.g_inversion_status = gtk.GtkLabel("")
        table.attach(self.g_inversion_status, 2, 3, 4, 5,
                     xoptions=gtk.FILL, yoptions=0)

        table.attach(gtk.GtkLabel(_("Toptone")), 4, 5, 0, 1,
                     xoptions=gtk.FILL, yoptions=0)
        self.g_toptone_box = gtk.GtkVBox()
        table.attach(self.g_toptone_box, 4, 5, 2, 3, xoptions=gtk.FILL)
        self.g_toptone_status = gtk.GtkLabel("")
        table.attach(self.g_toptone_status, 4, 5, 4, 5,
                     xoptions=gtk.FILL, yoptions=0)

        self.g_music_displayer = mpd.musicdisplayer.MusicDisplayer(utils.play_tone)
        self.g_music_displayer.set_usize(150, -1)
        hbox.pack_start(self.g_music_displayer)

        self.g_new = gu.bButton(self.action_area, _("New chord"),
                                self.new_question)
        self.g_repeat = gu.bButton(self.action_area, _("Repeat chord"),
                                lambda _o, self=self: self.m_t.play_question())
        self.g_repeat_arpeggio = gu.bButton(self.action_area,
              _("Repeat arpeggio"), 
              lambda _o, self=self: self.m_t.play_question_arpeggio())
        self.g_give_up = gu.bButton(self.action_area, _("Give up"),
                                    self.give_up)
        self.practise_box.show_all()
        ##############
        # config_box #
        ##############
        self.config_box.set_spacing(gnome.uiconsts.PAD_SMALL)
        self.g_lessonfile_selector = widgets.SelectLessonfileWidget(
            self.m_t, self.g_win, self.update_gui_after_lessonfile_change)
        self.g_lessonfile_selector.show()
        self.config_box.pack_start(self.g_lessonfile_selector)
        self.g_instrument_configurator  \
              = widgets.InstrumentConfigurator(self.m_exname, 3)
        self.config_box.pack_start(self.g_instrument_configurator, gtk.FALSE)
        self.g_instrument_configurator.show()
        self.update_gui_after_lessonfile_change()
    def update_answer_buttons(self, obj=None):
        for x in self.g_chordtype_box.children() \
              + self.g_inversion_box.children() \
              + self.g_toptone_box.children():
            x.destroy()
        if self.m_t.m_P:
            for type in self.m_t.m_P.m_chord_types.keys():
                b = gtk.GtkButton(self.m_t.m_P.m_chord_types[type])
                b.connect('clicked', self.on_type, type)
                self.g_chordtype_box.pack_start(b, gtk.FALSE)
                b.show()
            v = self.m_t.m_P.m_inversions
            v.sort()
            for x in v:
                if x == 0:
                    s = _("root position")
                else:
                    s = _("%i. inversion") % x
                b = gtk.GtkButton(s)
                b.connect('clicked', self.on_inversion, x)
                self.g_inversion_box.pack_start(b, gtk.FALSE)
                b.show()
            v = self.m_t.m_P.m_toptones
            v.sort()
            for x in v:
                b = gtk.GtkButton(str(x))
                b.connect('clicked', self.on_toptone, x)
                self.g_toptone_box.pack_start(b, gtk.FALSE)
                b.show()
    def update_gui_after_lessonfile_change(self):
        self.g_music_displayer.clear()
        if self.m_t.m_P:
            self.g_new.set_sensitive(gtk.TRUE)
        else:
            self.g_new.set_sensitive(gtk.FALSE)
        self.g_repeat.set_sensitive(gtk.FALSE)
        self.g_repeat_arpeggio.set_sensitive(gtk.FALSE)
        self.g_give_up.set_sensitive(gtk.FALSE)
        self.update_answer_buttons()
        self.g_win.set_title("Solfege - " + self.get_pretty_name())
    def on_type(self, button, type):
        if self.m_t.q_status in (const.QSTATUS_NO, const.QSTATUS_GIVE_UP):
            return
        if type == self.m_t.m_P.get_chordtype():
            self.g_type_status.set_text(_("Correct"))
            self.m_correct[0] = 1
        else:
            self.g_type_status.set_text(_("Wrong"))
            self.m_correct[0] = 0
    def on_toptone(self, button, toptone):
        if self.m_t.q_status in (const.QSTATUS_NO, const.QSTATUS_GIVE_UP):
            return
        if toptone == self.m_t.m_P.get_toptone():
            self.g_toptone_status.set_text(_("Correct"))
            self.m_correct[1] = 1
        else:
            self.g_toptone_status.set_text(_("Wrong"))
            self.m_correct[1] = 0
    def on_inversion(self, button, inversion):
        if self.m_t.q_status in (const.QSTATUS_NO, const.QSTATUS_GIVE_UP):
            return
        if inversion == self.m_t.m_P.get_inversion():
            self.g_inversion_status.set_text(_("Correct"))
            self.m_correct[2] = 1
        else:
            self.g_inversion_status.set_text(_("Wrong"))
            self.m_correct[2] = 0
    def new_question(self, widget=None):
        try:
            if self.m_t.new_question():
                self.g_give_up.set_sensitive(gtk.TRUE)
                self.g_repeat.set_sensitive(gtk.TRUE)
                self.g_repeat_arpeggio.set_sensitive(gtk.TRUE)
                self.g_type_status.set_text('')
                self.g_inversion_status.set_text('')
                self.g_toptone_status.set_text('')
                self.g_inversion_box.set_sensitive(self.m_t.m_P.get_inversion() != -1)
                self.g_toptone_box.set_sensitive(self.m_t.m_P.get_toptone() != -1)
                self.g_music_displayer.clear()
                self.m_t.play_question()
                self.m_correct = [0, 0, 0]
        except mpd.musicalpitch.InvalidNotenameException, n:
            gnome.ui.GnomeErrorDialog("In lessonfile '%s':\n Invalid notename: %s" % (
                self.get_string('lessonfile'), n.m_notename) )
            self.g_give_up.set_sensitive(gtk.FALSE)
            self.g_repeat.set_sensitive(gtk.FALSE)
            self.g_repeat_arpeggio.set_sensitive(gtk.FALSE)
    def on_end_practise(self):
        self.g_music_displayer.clear()
        self.g_repeat.set_sensitive(gtk.FALSE)
        self.g_repeat_arpeggio.set_sensitive(gtk.FALSE)
        self.g_give_up.set_sensitive(gtk.FALSE)
        self.m_t.q_status = const.QSTATUS_NO
        self.g_type_status.set_text('')
        self.g_inversion_status.set_text('')
        self.g_toptone_status.set_text('')
    def give_up(self, widget=None):
        if self.m_t.q_status == const.QSTATUS_NO:
            return
        clef = mpd.select_clef(self.m_t.m_P.get_music())
        fontsize = self.get_int('config/feta_font_size=20')
        self.g_music_displayer.display(r"\staff{\clef %s <%s>}" % (clef, self.m_t.m_P.get_music()), fontsize)
        self.g_type_status.set_text(self.m_t.m_P.get_chordtype())
        i = self.m_t.m_P.get_inversion()
        if i == 0:
            self.g_inversion_status.set_text(_("root position"))
        elif i > 0:
            self.g_inversion_status.set_text(_("%i. inversion") % i)
        i = self.m_t.m_P.get_toptone()
        if i != -1:
            self.g_toptone_status.set_text("%i" % i)
        self.m_t.give_up()



# GNU Solfege - eartraining for GNOME
# Copyright (C) 2000, 2001, 2002  Tom Cato Amundsen
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

import time, os, pickle
import utils
import configureoutput

STATISTICS_SAVE_DIR = os.path.join(os.path.expanduser('~'),
       '.solfege%s.%s' % (configureoutput.MAJOR_VERSION, configureoutput.MINOR_VERSION))
if not os.path.exists(STATISTICS_SAVE_DIR):
    os.mkdir(STATISTICS_SAVE_DIR)

YEAR = 0
MONTH = 1
DAY = 2
HOURS = 3
MINUTES = 4
SECONDS = 5
WEEKDAY = 6
JULIANDAY = 7

class AbstractStatistics:
    def __init__(self, teacher):
        self.m_t = teacher
        self.m_session_stat = {}
        self.m_today_stat = {}
        self.m_last7_stat = {}
        self.m_total_stat = {}
        self.m_dicts = {'session' : self.m_session_stat,
                        'today'   : self.m_today_stat,
                        'last7'   : self.m_last7_stat,
                        'total'   : self.m_total_stat}
        self.m_savepath = os.path.join(STATISTICS_SAVE_DIR, self.m_t.m_exname)
        if not os.path.exists(self.m_savepath):
            os.mkdir(self.m_savepath)
    def _add(self, question, answer):
        for D in [self.m_session_stat, self.m_today_stat,
                  self.m_last7_stat, self.m_total_stat]:
            if not D.has_key(question):
                D[question] = {}
            if not D[question].has_key(answer):
                D[question][answer] = 0
            D[question][answer] = D[question][answer] + 1
    def add_correct(self, answer):
        self._add(answer, answer)
    def add_wrong(self, question, answer):
        self._add(question, answer)
    def merge_stat_dicts(self, A, B):
        "Add content of B to A"
        for k in B.keys():
            if not A.has_key(k):
                A[k] = {}
            for n in B[k].keys():
                if not A[k].has_key(n):
                    A[k][n] = 0
                A[k][n] = A[k][n] + B[k][n]
        return A
    def load_statistics(self, datadir):
        for filename in os.listdir(datadir):
            t = int(filename)
            lt = time.localtime(t)
            f = open(os.path.join(datadir, filename), 'r')
            D = pickle.load(f)
            f.close()
            now = int(time.time())
            lt_today = time.localtime(time.time())
            if lt[YEAR] == lt_today[YEAR] and \
                    lt[JULIANDAY] == lt_today[JULIANDAY]:
                self.m_today_stat = self.merge_stat_dicts(self.m_today_stat, D)
            if now - t < 60*60*24*7: # 7 days
                self.m_last7_stat = self.merge_stat_dicts(self.m_last7_stat, D)
            self.m_total_stat = self.merge_stat_dicts(self.m_total_stat, D)
    def get(self):
        """Will return a 0 <= value <= 1.0 that say how many percent is
        correct in this session
        """
        c = t = 0
        for k in self.m_session_stat.keys():
            if self.m_session_stat[k].has_key(k):
                c = c + self.m_session_stat[k][k]
            t = t + self.get_num_guess(self.m_session_stat, k)
        if t > 0:
            return 1.0 * c / t
        else:
            return 0.0
    def display(self):
        print
        v = self.m_session_stat.keys()
        v.sort()
        for x in v:
            print x, self.m_session_stat[x]
    def get_keys(self, all=0):
        """
        by default it returns the keys for all questions that have
        been asked. If 'all' is true, it also includes the keys for
        all the wrong answers.
        """
        keys = []
        for st in self.m_session_stat, self.m_today_stat, self.m_last7_stat, self.m_total_stat:
            for k in st.keys():
                if k not in keys:
                    keys.append(k)
                if all:
                    for wk in self.m_total_stat[k].keys():
                        if wk not in keys:
                            keys.append(wk)
        keys.sort()
        return keys
    def get_num_guess(self, st, key):
        if not st.has_key(key):
            return 0
        t = 0
        for i in st[key].keys():
            t = t + st[key][i]
        return t
    def get_percentage_correct(self, st, key):
        """
        This was added to be used for harmonic-intervall.
        """
        if not st.has_key(key):
            return 0.0
        if st[key].has_key(key):
            num_correct = st[key][key]
        else:
            num_correct = 0
        total = 0
        for n in st[key].keys():
            total = total + st[key][n]
        return num_correct * 100.0 / total
    def key_to_pretty_name(self, k):
        return k
    def get_label_style(self):
        return 'normal'
    def reset_session(self):
        self.m_session_stat = {}


class Statistics(AbstractStatistics):
    def __init__(self, teacher):
        AbstractStatistics.__init__(self, teacher)
        self.load_statistics(self.m_savepath)
    def save_data(self):
        if self.m_session_stat:
            f = open(os.path.join(self.m_savepath, str(int(time.time()))), 'w')
            pickle.dump(self.m_session_stat, f)
            f.close()


class LessonStatistics(AbstractStatistics):
    def __init__(self, teacher):
        AbstractStatistics.__init__(self, teacher)
        self.m_cur_collection = self.m_cur_file = None
        self.lessonfile_changed(self.m_t.get_string('lessoncollection'),
                                self.m_t.get_string('lessonfile'))
    def get_label_style(self):
        return self.m_t.m_P.header.labelformat
    def lessonfile_changed(self, new_collection, new_file):
        if not new_file:
            return
        if self.m_cur_file and self.m_session_stat != {}:
            self.save_data()

        self.m_session_stat = {}
        self.m_today_stat = {}
        self.m_last7_stat = {}
        self.m_total_stat = {}
        self.m_cur_collection = new_collection
        self.m_cur_file = new_file
        self.create_statistics_dir()

        # if the lessonfile has changed, we have to
        # delete all statistics just to be save.
        if self.get_hash_of_statistics() != self.get_hash_of_lessonfile():
            p = os.path.join(self.m_savepath,
                             self.m_cur_collection,
                             self.m_cur_file)
            if os.path.isfile("%s_hash" % p):
                os.remove("%s_hash" % p)
            for f in os.listdir(p):
                os.remove(os.path.join(p, f))
        self.load_statistics(os.path.join(self.m_savepath,
                                          self.m_cur_collection,
                                          self.m_cur_file))
    def save_data(self):
        if self.m_session_stat == {}:
            return
        #save the hashvalue for the lessonfile this statistics was made with
        f = open(os.path.join(self.m_savepath,
                              self.m_cur_collection,
                              '%s_hash' % self.m_cur_file), 'w')
        f.write(str(self.get_hash_of_lessonfile()))
        f.close()
        f = open(os.path.join(self.m_savepath,
                              self.m_cur_collection,
                              self.m_cur_file,
                              str(int(time.time()))), 'w')
        pickle.dump(self.m_session_stat, f)
        f.close()
    def create_statistics_dir(self):
        dir = os.path.join(self.m_savepath, self.m_cur_collection)
        if not os.path.exists(dir):
            os.mkdir(dir)

        dir = os.path.join(dir, self.m_cur_file)
        if not os.path.exists(dir):
            os.mkdir(dir)
    def get_hash_of_statistics(self):
        # !wrong function name!
        # get the hash for the content of the lessonfile that was used last
        # time statistics was saved
        if os.path.isfile(os.path.join(self.m_savepath,
                                       self.m_cur_collection,
                                       '%s_hash' % self.m_cur_file)):
            s = open(os.path.join(self.m_savepath,
                                         self.m_cur_collection,
                                         '%s_hash' % self.m_cur_file)).read()
            try:
                return int(s)
            except:
                return 0
        else:
            return 0
    def get_hash_of_lessonfile(self):
        return hash(open(os.path.expanduser(os.path.join(
            self.m_t.get_string('lessoncollections/%s' % self.m_cur_collection),
            self.m_cur_file))).read())


class IntervallStatistics(Statistics):
    def key_to_pretty_name(self, key):
        return utils.int_to_intervallname(key, 1, 1)

class HarmonicIntervallStatistics(Statistics):
    def key_to_pretty_name(self, key):
        return utils.int_to_intervallname(key, 1, 0)



# GNU Solfege - eartraining for GNOME
# Copyright (C) 2000, 2001, 2002  Tom Cato Amundsen
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

"""
The Intervall class is a class that is used to do math with
intervalls and musical pitches. You use this class if you
need to know the difference between a augmented second and
a minor third. If you don't require this you can probably
ise the simpler function utils.int_to_intervallname

>>> from musicalpitch import MusicalPitch
>>> a=MusicalPitch("c'")
>>> b=Intervall("m2")
>>> (a+b).str()
"des'"
>>> for t in ('d', 'm', 'M', 'a'):
...     for n in ('2', '3', '6', '7', '9', '10'):
...         i = Intervall(t+n)
...         assert i.get_short_name() == t+n
"""
import string, re

class Intervall:
    """
    The intervall is internally a intervall less than octave
    pluss n octaves. The data variables:
      m_dir
      m_octave
      m_intervall
      m_mod
    should NOT be touched by anyone, except MusicalPitch.__add__
    """
    def __init__(self, iname=None):
        self.m_dir = 1 # value as to be 1 or -1 for initialised obj
        self.m_octave = 0 # 0, 1, 2, 3 etc
        self.m_intervall = 0 # 0:unison, 1:seond, ... 6: septim
        # unison:              dim   perfect   aug
        # second:  -2:dim -1:minor 0:major   1:aug
        # third:      dim    minor   major     aug
        # fourth:              dim   perfect   aug
        # fifth:               dim   perfect   aug
        # sixth:      dim    minor   major     aug
        # seventh:    dim    minor   major     aug
        self.m_mod = 0
        if iname:
            self.set_from_string(iname)
    def errorcheck(self):
        assert 0 <= self.m_intervall <= 6
        assert -2 <= self.m_mod <= 1 # should be increased to -3 <= x <= 2
        assert self.m_octave >= 0
        assert self.m_dir in (-1, 1)
    def _set(self, dir, intervall, mod, octave):
        self.m_dir = dir
        self.m_intervall = intervall
        self.m_mod = mod
        self.m_octave = octave
        if __debug__:
            self.errorcheck()
        return self
    def set_from_int(self, i):
        """It returns self to allow chaining: set_from_int(4).pretty_name()
        """
        if i < 0:
            self.m_dir = -1
        else:
            self.m_dir = 1
        self.m_octave = abs(i) / 12
        self.m_mod, self.m_intervall = (
               (0, 0),          # unison
               (-1, 1), (0, 1), # second
               (-1, 2), (0, 2), # third
               (0, 3),          # fourth
               (-1, 4),         # dim 5, tritonus
               (0, 4),          # fifth
               (-1, 5), (0, 5), # sixth
               (-1, 6), (0, 6))[abs(i) % 12] # seventh
        return self
    def set_from_string(self, s):
        """
        second  m2 M2
        third   m3 M3
        fourth  4
        fifth   d5 5
        sixth   m6 M6
        seventh m7 M7
        octave  8
        none    m9 M9
        decim   m10 M10
        """
        s = string.strip(s)
        if s[0] == "-":
            self.m_dir = -1
            s = s[1:]
        else:
            self.m_dir = 1
        n, i = re.match("(m|M|d|a*)(\d*)", s).groups()
        if int(i) <= 7:
            self.m_intervall = int(i) - 1
            self.m_octave = 0
        else:
            self.m_intervall = int(i) - 8
            self.m_octave = 1
        if self.m_intervall in (1, 2, 5, 6):
            self.m_mod = {'d': -2, 'm': -1, 'M': 0, 'a': 1}[n]
        elif self.m_intervall in (4, 5):
            self.m_mod = {'d': -1, 'p': 0, '': 0, 'a': 1}[n]
    def get_intvalue(self):
        if __debug__:
            self.errorcheck()
        return ([0, 2, 4, 5, 7, 9, 11][self.m_intervall] + self.m_octave * 12 + self.m_mod) * self.m_dir
    def __str__(self):
        if __debug__:
            self.errorcheck()
        s = "(Intervall %i %imod %io" % (self.m_intervall, self.m_mod, self.m_octave)
        if self.m_dir == -1:
            s = s + " down)"
        else:
            s = s + " up)"
        return s
    def get_short_name(self):
        """
        DON'T USE THIS FUNCTION ANYWHERE!!!!!!!!
        The format for the names will change without notice, so the
        return value of this function is *ONLY* human readable.
        DON'T LET ANY CODE DEPEND ON THE FORMAT OF THE RETURN VALUE
        """
        n = ""
        if self.m_octave == 1 and self.m_intervall in (1, 2):
            n = n + {-2:'d', -1:'m', 0:'M', 1:'a'}[self.m_mod]
            n = n + {1: '9', 2:'10'}[self.m_intervall]
            if self.m_dir == -1:
                n = n + " down"
            return n
        elif self.m_octave == 1 and self.m_intervall == 0:
            n = n + {-1:'d', 0:'', 1:'a'}[self.m_mod]
            n = n + "8"
            if self.m_dir == -1:
                n = n + " down"
            return n
        elif self.m_intervall in (1, 2, 5, 6):
            n = n + {-2:'d', -1:'m', 0:'M', 1:'a'}[self.m_mod]
            n = n + {1:'2', 2:'3', 5:'6', 6:'7'}[self.m_intervall]
        elif self.m_intervall in (0, 3, 4, 7):
            n = n + {-1:'d', 0:'', 1:'a'}[self.m_mod]
            n = n + {0:'1', 3:'4', 4:'5', 7:'8'}[self.m_intervall]
        if self.m_octave > 0:
            n = n + "+%i octave" % self.m_octave
        if self.m_dir == -1:
            n = n + " down"
        return n

def _test():
    import doctest
    return doctest.testmod(intervall)

if __name__=="__main__":
    _test()
    import sys
    sys.exit()

if __name__=="__main__":
    import musicalpitch
    print "testing the intervall module"
    i = Intervall()
    for iv, intervall, modifier, octave, dir in (
          (5, 3, 0, 0, 1), (1, 1, -1, 0, 1), (2, 1, 0, 0, 1),
          (-1, 1, -1, 0, -1), (-2, 1, 0, 0, -1)):
        i.set_from_int(iv)
        print "set_from_int(%i)" % iv
        if i.m_intervall == intervall:
            print "intervall:%i, " % intervall,
        else:
            print "intervall: failed:%i!=%i, " % (i.m_intervall, intervall),
        if i.m_mod == modifier:
            print "mod:%i, " % modifier,
        else:
            print "mod: failed:%i=!%i, " % (i.m_mod, modifier),
        if i.m_octave == octave:
            print "octave:%i, " % octave,
        else:
            print "octave: failed:%i=!%i, " % (i.m_octave, octave),
        if i.m_dir == dir:
            print "dir:%i, " % dir,
        else:
            print "dir: failed:%i=!%i, " % (i.m_dir, dir),
        print
    for old, new, i in (("c'", "d'", 2), ("e'", "fis'", 2), ("e", "b,", -5),
           ("e", "bes,", -6), ("e", "ais,", -6)):
        n = musicalpitch.MusicalPitch(old) + Intervall().set_from_int(i)
        if n.str() == new:
            print "ok: %s + %i == %s" % (old, i, new)
        else:
            print "%s + %i == %s failed:" % (old, i, new), n.str()
    for x in ('m2', 'M2', 'm3', 'M3', '4', 'd5', '5', 'm6', 'M6',
              'm7', 'M7', '8', 'm9', 'M9', 'm10', 'M10'):
        i = Intervall()
        i.set_from_string(x)
        print x, i
